/***************************************************************************
 *   Copyright (C) 2005 by Alexander Shiyan                                *
 *   shc@users.sourceforge.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qdir.h>
#include <qimage.h>
#include <qstring.h>
#include <qtextcodec.h>
#include <qfiledialog.h>
#include <qtranslator.h>
#include <qapplication.h>
#include <qsplashscreen.h>

#ifdef ENABLE_BINRELOC
#	include "prefix.h"
#	define	BR_WRAPDATA(a,b)	BR_DATADIR(QString(a).append("/").append(b).ascii())
#	define	BR_WRAPPREFIX(a)	BR_PREFIX(QString("/").append(a).ascii())
#else
#	if defined(Q_WS_WIN)
#		define	BR_WRAPDATA(a,b)	b
#		define	BR_WRAPPREFIX(a)	QString("..\\").append(a)
#	else
#		warning "Compile on this target is untested!"
#	endif
#endif

#if defined(Q_WS_X11)
#	include <signal.h>
#endif

#include "config.h"
#include "utils.h"
#include "tareas.h"
#include "tconfig.h"

#include "mainwindow.h"

TConfig *Config;
TAreas *Areas;
QSplashScreen *Splash = NULL;

void StartUpMessage(bool useGUI, QString mess)
{
	if (useGUI)
		informationmessage(NULL, mess);
	else
		fprintf(stdout, mess.append("\n").local8Bit());
}

int main(int argc, char ** argv)
{
	int i, ret = 1, fres;
	bool ffound = false, useGUI = true, showsplash = true, allowroot = false;
	QString UserHome = QString::null;

#if defined(Q_WS_X11)
	useGUI = !QString(getenv("DISPLAY")).isEmpty();
#endif

	qApp = new QApplication(argc, argv, useGUI);

	QTranslator translator(0);
	QDir tsdir = QDir(BR_WRAPDATA("/qfe", "translations")), dir = QDir(BR_WRAPPREFIX("src"));
	if (dir.cd("ts", false))
		tsdir = dir;
	translator.load(QString("qfe_") + QTextCodec::locale(), tsdir.absPath());
	qApp->installTranslator(&translator);

	tsdir.cdUp();

	for (i = 1; i < qApp->argc(); i++)
	{
		QString arg = qApp->argv()[i];
		if ((arg == "-c") || (arg == "--configdir"))
		{
			if (qApp->argc() > (i + 1))
			{
				UserHome = qApp->argv()[i + 1];
				i++;
			} else {
				StartUpMessage(useGUI, QObject::tr("Option '--configdir' require path as parameter."));
				return 1;
			}
		} else if ((arg == "-ns") || (arg == "--nosplash"))
		{
			showsplash = false;
		} else if ((arg == "-V") || (arg == "--version"))
		{
			StartUpMessage(useGUI, "QFE - " + QObject::tr("Qt FTN Message Editor") + "\n" + QString(QObject::tr("Version") + ": %1").arg(QFE_VERSION));
			return 0;
		} else if (arg == "--root")
		{
			allowroot = true;
		} else
			StartUpMessage(false, QObject::tr("Incorrect option") + " " + arg);
	}

	if (!useGUI)
	{
		StartUpMessage(false, QObject::tr("Program must be run only in graphic mode."));
		return 1;
	}

	QPixmapCache::setCacheLimit(512);

	Config = new TConfig(&tsdir, UserHome);

#if defined(Q_WS_X11)
	pid_t qfepid = getpid();
	QFile PID(Config->ConfigPath + "qfe.pid");
	if (PID.exists())
	{
		if (FileOpenFunc(&PID, IO_ReadOnly))
		{
			if (PID.readBlock((char*)&qfepid, sizeof(qfepid)) == sizeof(qfepid))
				if (kill(qfepid, 0) == 0)
					ffound = true;
			FileCloseFunc(&PID);
		} else
			ffound = true;
	} else
		if (FileOpenFunc(&PID, IO_WriteOnly))
		{
			PID.writeBlock((char*)&qfepid, sizeof(qfepid));
			FileCloseFunc(&PID);
		}

	if (ffound)
	{
		informationmessage(NULL, QObject::tr("QFE alredy running."));
		return 1;
	}
#endif

#if defined(Q_WS_X11)
	if ((getuid()) == 0 && !allowroot)
	{
		informationmessage(NULL, QObject::tr("This program must run as user."));
		return 1;
	}
#else
	Q_UNUSED(allowroot);
#endif

	ffound = false;

	QFileDialog *fd = new QFileDialog(NULL);
	fd->setFilter(QObject::tr("All Files (*)"));
	fd->addFilter(QObject::tr("Configuration Files (*.cfg)"));
	fd->setSelectedFilter(0);
	fd->setCaption(QObject::tr("Select file..."));
	fd->setIcon(*Config->FindPixmap("icon_load.xpm"));
	fd->setMode(QFileDialog::ExistingFile);
	fd->setInfoPreviewEnabled(true);

	while (!ffound)
	{
		if (!testexists(Config->GetStr(CONFIG_SECTION_FIDO, CONFIG_FIDOCONFNAME, getConfigFileName())))
		{
			if (questionmessage(NULL, QObject::tr("Config-file not found. Do You want select Config-file now?")) == QMessageBox::Yes)
			{
				if (fd->exec() == QDialog::Accepted)
					Config->SetStr(CONFIG_SECTION_FIDO, CONFIG_FIDOCONFNAME, fd->selectedFile());
			} else
				break;
		} else {
			fres = Config->InitFido();
			if (fres == ERROR_INITFIDO_CONFIG)
			{
				Config->DoneFido();
				if (questionmessage(NULL, QObject::tr("Can't open config. Do You want select Config-file now?")) == QMessageBox::Yes)
				{
					if (fd->exec() == QDialog::Accepted)
						Config->SetStr(CONFIG_SECTION_FIDO, CONFIG_FIDOCONFNAME, fd->selectedFile());
				} else
					break;
			} else if (fres == ERROR_INITFIDO_LOCK)
			{
				Config->DoneFido();
				errormessage(NULL, QObject::tr("Can't create lock file.") + " " + QObject::tr("Program terminated."));
				break;
			} else if (fres == ERROR_INITFIDO_SMAPI)
			{
				Config->DoneFido();
				errormessage(NULL, QObject::tr("Smapi API open error.") + " " + QObject::tr("Program terminated."));
				break;
			} else if (fres == ERROR_INITFIDO_SMAPI_OLD)
			{
				Config->DoneFido();
				errormessage(NULL, QObject::tr("Smapi library too old.") + " " + QObject::tr("Program terminated."));
				break;
			} else
				ffound = true;
		}
	}
	delete fd;

	if (ffound)
	{
		if ((Config->ScreenRect.width() < 800) || (Config->ScreenRect.height() < 600))
		{
			errormessage(NULL, QObject::tr("Program work with sceeen Size not lower 800x600."));
		} else {
			if (showsplash)
			{
				QImage SplashImg = QFE_LOGO.convertToImage();
				Splash = new QSplashScreen(QPixmap(SplashImg.smoothScale((int)((double)SplashImg.width() * 1.5), (int)((double)SplashImg.height() * 1.5), QImage::ScaleMin)));
				Splash->show();
				Splash->message(QObject::tr("Initializing..."), Qt::AlignLeft, Qt::white);
				QTimer::singleShot(1650, Splash, SLOT(hide()));
			}

			qApp->setMainWidget(new TMainWindow());

			qApp->mainWidget()->setCaption(QObject::tr("Qt FTN Message Editor"));

			qApp->connect(qApp, SIGNAL(lastWindowClosed()), qApp, SLOT(quit()));

			qApp->mainWidget()->show();
			ret = qApp->exec();

			if (Splash != NULL)
				delete Splash;

			delete qApp->mainWidget();
		}
		Config->DoneFido();
	}

	delete Config;

#if defined(Q_WS_X11)
	PID.remove();
#endif
	return ret;
}
