/***************************************************************************
 *   Copyright (C) 2005 by Alexander Shiyan                                *
 *   shc@users.sourceforge.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tarea.h"

extern TConfig *Config;

TArea::TArea(TAreas *_Areas, s_area Area, uint _Type)
{
	Areas = _Areas;
	Name = Area.areaName;
	Path = Area.fileName;
	Aka = addr2str2(*Area.useAka);
	Desc = Config->toUTF(Area.description);
	MsgbType = Area.msgbType;

	Uplink = QString::null;
	UplinkPwd = QString::null;
	UplinkRbt = "AREAFIX";

	for (uint i = 0; i < Area.downlinkCount; i++)
		if (Area.downlinks[i]->defLink)
		{
			Uplink = addr2str2(Area.downlinks[i]->link->hisAka);
#ifdef FIDOCONF_VER1_4
			if (Area.downlinks[i]->link->AreaFix != 0)
			{
				UplinkPwd = Area.downlinks[i]->link->areaFixPwd;
#else
			if (Area.downlinks[i]->link->areafix.on)
			{
				UplinkPwd = Area.downlinks[i]->link->areafix.pwd;
				if (Area.downlinks[i]->link->areafix.name)
					UplinkRbt = Area.downlinks[i]->link->areafix.name;
#endif
			} else
				UplinkPwd = Area.downlinks[i]->link->defaultPwd;
		}

	Group = Area.group;
	Handle = NULL;
	Type = _Type;

	setAutoDelete(true);

	dummyLastRead = 0;

	Rescan();
}

TArea::~TArea()
{
	Close();
}

bool TArea::Open(bool DoLock)
{
	if (Handle == NULL)
	{
		Handle = MsgOpenArea((byte*)Path.ascii(), ((MsgbType & MSGTYPE_STORAGES) == MSGTYPE_SDM) ? MSGAREA_NORMAL : MSGAREA_CRIFNEC, MsgbType);
		if ((Handle != NULL) && DoLock)
			MsgLock(Handle);
	}
	return (Handle != NULL);
}

void TArea::Close()
{
	if (Handle != NULL)
	{
		for (uint i = 0; i < count(); i++)
			at(i)->Close();
		MsgUnlock(Handle);
		MsgCloseArea(Handle);
		Handle = NULL;
	}
}

uint TArea::Marked()
{
	uint i, ret = 0;

	for (i = 0; i < count(); i++)
	{
		TMessageItem *It = at(i)->MessageItem;
		if (It != NULL)
			if (It->isMarked())
				ret++;
	}
	return ret;
}

TMessage *TArea::Add(uint idx)
{
	TMessage *It = new TMessage(this, idx);
	append(It);
	return It;
}

TMessage *TArea::New()
{
	TMessage *ret = NULL;

	Close();
	if (Open(false))
	{
		ret = new TMessage(this, 0, true);
		append(ret);
		Close();
	}
	return ret;
}

void TArea::KillMessage(uint idx)
{
	if (Open())
	{
		MsgKillMsg(Handle, idx);
		Rescan();
	}
}

uint TArea::GetLastRead()
{
	uint val, ret = dummyLastRead;
	QFile lastread;

	switch (MsgbType)
	{
		case MSGTYPE_JAM:
			lastread.setName(Path + ".jlr");
			break;
		case MSGTYPE_SQUISH:
			lastread.setName(Path + ".sql");
			break;
		case MSGTYPE_SDM:
			lastread.setName(Path + QDir::separator() + "lastread");
			break;
		default:
			return dummyLastRead;
	}

	if (lastread.exists())
		if (FileOpenFunc(&lastread, IO_ReadOnly))
		{
			if (lastread.readBlock((char*)&val, sizeof(val)) == sizeof(val))
				ret = val;
			FileCloseFunc(&lastread);
		}

	return QMIN(ret, count());
}

void TArea::SetLastRead(uint idx)
{
	QFile lastread;

	dummyLastRead = idx;

	switch (MsgbType)
	{
		case MSGTYPE_JAM:
			lastread.setName(Path + ".jlr");
			break;
		case MSGTYPE_SQUISH:
			lastread.setName(Path + ".sql");
			break;
		case MSGTYPE_SDM:
			lastread.setName(Path + QDir::separator() + "lastread");
			break;
		default:
			return;
	}

	if (FileOpenFunc(&lastread, IO_WriteOnly))
	{
		lastread.writeBlock((char*)&idx, sizeof(idx));
		FileCloseFunc(&lastread);
	}
}

uint TArea::Index2UID(const uint idx)
{
	uint ret = 0;

	if (Open())
	{
		ret = MsgMsgnToUid(Handle, idx);
		Close();
	}
	return ret;
}

uint TArea::UID2Index(const uint uid)
{
	int ret = 0;

	if (Open())
	{
		ret = MsgUidToMsgn(Handle, uid, UID_EXACT);
		Close();
	}
	return ((ret > 0) ? (uint)ret : 0);
}

void TArea::Rescan()
{
	Close();
	clear();
	Unread = 0;
	if (Open())
	{
		uint Cnt = MsgGetNumMsg(Handle);
		for (uint i = 1; i <= Cnt; i++)
			if (!Add(i)->isFlagsSet(MSGREAD))
				Unread++;
		Close();
	}
}

void TArea::MarkAllAsRead()
{
	if (Open())
	{
		for (uint i = 0; i < count(); i++)
			at(i)->MarkAsRead();
		Close();
	}
}
