/***************************************************************************
 *   Copyright (C) 2005 by Alexander Shiyan                                *
 *   shc@users.sourceforge.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tconfig.h"

#include "config.h"
#include "stdconfig.h"

#define CONFIG_VERSION		"Version"

#define CONFIG_DRAFTAREA	"DraftArea"

#define CONFIG_WINDOWPOSX	"%1PosX"
#define CONFIG_WINDOWPOSY	"%1PosY"
#define CONFIG_WINDOWWIDTH	"%1Width"
#define CONFIG_WINDOWHEIGHT	"%1Height"

#define CONFIG_LISTWIDTH	"%1Width%2"
#define CONFIG_LISTSORT		"%1Sort"
#define CONFIG_LISTORDER	"%1Order"

#define CONFIG_TAGCOLOR		"%1Color"
#define CONFIG_TAGSTYLE		"%1Style"

#define CONFIG_ACCEL		"Accel%1"

#define CONFIG_TOOLBAR_SHOW	"%1ToolBarView"
#define CONFIG_TOOLBAR_DOCK	"%1ToolBarDock"
#define CONFIG_TOOLBAR_ORIENT	"%1ToolBarOrient"
#define CONFIG_TOOLBAR_POSX	"%1ToolBarPosX"
#define CONFIG_TOOLBAR_POSY	"%1ToolBarPosY"

#define CONFIG_LIST_COUNTER	"%1Count"
#define CONFIG_LIST_ITEM	"%1Item%2"

TConfig::TConfig(QDir *Home, QString UserHome)
{
	FidoConfLock = 0;

	QDir CfgRoot = QDir::homeDirPath();
	if (UserHome.isEmpty())
	{
		CfgRoot.mkdir(".qfe", false);
		CfgRoot.cd(".qfe", false);
	} else
		CfgRoot = QDir(UserHome);

	// This will be removed in latest versions
	if (CompareVersions(QFE_VERSION, "0.3") > 0)
		CfgRoot.rename("qferc", "qfe.ini", false);
	//

	ConfigPath = CfgRoot.absPath() + QDir::separator();
	Settings = new TIniSettings(ConfigPath + "qfe.ini");
	HomePath = Home->absPath();
	ScreenRect = QDesktopWidget().screenGeometry(0);

	SetStr(QString::null, CONFIG_VERSION, QFE_VERSION);

	CheckTemplates();
	TextCodec = QTextCodec::codecForName(GetStr(CONFIG_SECTION_FIDO, CONFIG_FIDOCODEPAGE, "IBM 866"));
	FontDatabase = new QFontDatabase();
	AddressBook = new TBook(ConfigPath + ADDRESSBOOK_NAME);
}

TConfig::~TConfig()
{
	delete Settings;
	delete FontDatabase;
	delete AddressBook;
}

void TConfig::RestoreFile(QString Name)
{
	QByteArray data = qembed_findData(Name.ascii());

	if (data.size() > 0)
	{
		QFile f(ConfigPath + Name);

		if (f.exists())
		{
			QDir d(ConfigPath);
			QString OldName = Name + ".old";
			if (d.exists(OldName, false))
				d.remove(OldName, false);
			d.rename(Name, OldName, false);
			fprintf(stdout, "File %s renamed to %s due update.\n", Name.ascii(), OldName.ascii());
		}

		if (FileOpenFunc(&f, IO_Raw | IO_WriteOnly))
		{
			f.writeBlock(data.data(), data.size());
			FileCloseFunc(&f);
		}
	}
}

void TConfig::FlushIni()
{
	Settings->Flush();
}

void TConfig::CheckTemplates()
{
	if (!testexists(ConfigPath + TPL_MESSAGE_TPL))
		RestoreFile(TPL_MESSAGE_TPL);
	if (!testexists(ConfigPath + TPL_REPLY_TPL))
		RestoreFile(TPL_REPLY_TPL);
	if (!testexists(ConfigPath + TPL_FORWARD_TPL))
		RestoreFile(TPL_FORWARD_TPL);
	if (!testexists(ConfigPath + TPL_UNSUBSCRIBE_TPL))
		RestoreFile(TPL_UNSUBSCRIBE_TPL);

	// CompareVersions will be removed in latest versions
	if (!testexists(ConfigPath + TPL_SAVETXT_TPL) || (CompareVersions(QFE_VERSION, "0.3-cvs") > 0))
		RestoreFile(TPL_SAVETXT_TPL);
	//

	if (!testexists(ConfigPath + TPL_SAVEHTM_TPL))
		RestoreFile(TPL_SAVEHTM_TPL);

	if (!testexists(ConfigPath + TPL_ORIGIN_LST))
		RestoreFile(TPL_ORIGIN_LST);
	if (!testexists(ConfigPath + TPL_KLUDGES_LST))
		RestoreFile(TPL_KLUDGES_LST);
}

uint TConfig::InitFido()
{
	QString fcf = GetStr(CONFIG_SECTION_FIDO, CONFIG_FIDOCONFNAME, getConfigFileName());

	setvar("module", "qfe");
	FidoConfig = readConfig(fcf.isEmpty() ? NULL : fcf.ascii());

	if (FidoConfig == NULL)
		return ERROR_INITFIDO_CONFIG;

	if (FidoConfig->lockfile)
	{
		FidoConfLock = lockFile((const char*)FidoConfig->lockfile, FidoConfig->advisoryLock);
		if (FidoConfLock < 0)
		{
			disposeConfig(FidoConfig);
			FidoConfig = NULL;
			return ERROR_INITFIDO_LOCK;
		}
	}

	Smapi.req_version = 2;
	Smapi.def_zone = GetInt(CONFIG_SECTION_FIDO, CONFIG_FIDODEFZONE, 2);
	if (MsgOpenApi(&Smapi) == 0)
	{
		if ((Smapi.smapi_version == 2) && (Smapi.smapi_subversion >= 0x240))
			return 0;
		else
			return ERROR_INITFIDO_SMAPI_OLD;
	} else
		return ERROR_INITFIDO_SMAPI;
}

void TConfig::DoneFido()
{
	MsgCloseApi();
	if (FidoConfig != NULL)
	{
		if (FidoConfig->lockfile && (FidoConfLock != 0))
		{
			FreelockFile(FidoConfig->lockfile, FidoConfLock);
			FidoConfLock = 0;
		}
		disposeConfig(FidoConfig);
		FidoConfig = NULL;
	}
}

QString TConfig::CharsetForMessages()
{
	QString tmp = GetStr(CONFIG_SECTION_FIDO, CONFIG_FIDOCODEPAGE, "IBM 866");
	for (uint i = 0; i < CODEPAGES_COUNT; i++)
		if (tmp == QFECodePages[i][0])
			return (QFECodePages[i][2]);
	return "IBMPC 2";
}

QPixmap *TConfig::FindPixmap(QString Name)
{
	static QPixmap p;
	QPixmap tmp;

	if (QPixmapCache::find(Name, p))
		if (!p.isNull())
			return &p;
	tmp = QPixmap::fromMimeSource(Name);
	if (!tmp.isNull())
	{
		if (!QPixmapCache::insert(Name, tmp))
			fprintf(stdout, "Fail to insert pixmap in cache. Please report to developers.\n");
		p = tmp;
		return &p;
	}

	fprintf(stdout, "Pixmap %s not found. Please report to developers.\n", Name.ascii());

	return NULL;
}

QString TConfig::GetSection(QString Section)
{
	return (Section.isEmpty() ? CONFIG_SECTION_MAIN : Section);
}

void TConfig::ChangeTextCodec(QString Value)
{
	if (Value != QString(TextCodec->name()))
		TextCodec = QTextCodec::codecForName(Value);
}

QString TConfig::toUTF(char *Value, uint ValueSize)
{
	return TextCodec->toUnicode((ValueSize == 0) ? QCString(Value) : QCString(Value, ValueSize));
}

QCString TConfig::fromUTF(QString Value)
{
	return TextCodec->fromUnicode(Value);
}

int TConfig::GetInt(QString Section, QString Key, int Default)
{
	return Settings->GetString(GetSection(Section), Key, QString("%1").arg(Default)).toInt();
}

bool TConfig::GetBool(QString Section, QString Key, bool Default)
{
	return strcompare(Settings->GetString(GetSection(Section), Key, Default ? "true" : "false"), "true");
}

QString TConfig::GetStr(QString Section, QString Key, QString Default)
{
	return Settings->GetString(GetSection(Section), Key, Default);
}

void TConfig::GetStrList(QString Section, QString ListName, QStringList *Value)
{
	int i, cnt = GetInt(Section, QString(CONFIG_LIST_COUNTER).arg(ListName), 0);
	QString tmp;

	Value->clear();
	for (i = 0; i < cnt; i++)
	{
		tmp = GetStr(Section, QString(CONFIG_LIST_ITEM).arg(ListName).arg(i + 1), QString::null).stripWhiteSpace();
		if (!tmp.isEmpty())
			Value->append(tmp);
	}
}

QString TConfig::GetDraftArea()
{
	ps_area Area = NULL;
	QString tmp = GetStr(QString::null, CONFIG_DRAFTAREA, QString::null);

	if (!tmp.isEmpty())
		Area = getArea(FidoConfig, (char*)tmp.ascii());

	return ((Area == NULL) ? QString::null : tmp);
}

void TConfig::GetPos(QWidget *Wnd)
{
	Wnd->move(GetInt(QString::null, QString(CONFIG_WINDOWPOSX).arg(Wnd->name()), Wnd->pos().x()), GetInt(QString::null, QString(CONFIG_WINDOWPOSY).arg(Wnd->name()), Wnd->pos().y()));
}

void TConfig::GetPos(QWidget *Wnd, int DefX, int DefY)
{
	int x = GetInt(QString::null, QString(CONFIG_WINDOWPOSX).arg(Wnd->name()), DefX);
	int y = GetInt(QString::null, QString(CONFIG_WINDOWPOSY).arg(Wnd->name()), DefY);
	while (x > ScreenRect.width())
		x -= 10;
	while (y > ScreenRect.height())
		y -= 10;
	Wnd->move(x, y);
}

void TConfig::GetSize(QWidget *Wnd, int DefW, int DefH)
{
	Wnd->resize(QMIN(ScreenRect.width(), GetInt(QString::null, QString(CONFIG_WINDOWWIDTH).arg(Wnd->name()), DefW)), QMIN(ScreenRect.height(), GetInt(QString::null, QString(CONFIG_WINDOWHEIGHT).arg(Wnd->name()), DefH)));
}

void TConfig::GetSplitter(QString Section, QSplitter *Value, const int *Default)
{
	int i;
	size_t j;
	QValueList<int> list = Value->sizes();

	for (j = 0; j < list.size(); j++)
	{
		i = GetInt(Section, QString("%1%2").arg(Value->name()).arg(j), Default ? *(Default + j) : -1);
		if (i == -1) return;
		QValueList<int>::Iterator it = list.at(j);
		*it = i;
	}
	Value->setSizes(list);
}

void TConfig::GetList(QString Section, QListView *Value, const int *Default)
{
	for (int i = 0; i < Value->columns(); i++)
		Value->setColumnWidth(i, GetInt(Section, QString(CONFIG_LISTWIDTH).arg(Value->name()).arg(i), Default ? *(Default + i) : Value->columnWidth(i)));
	Value->setSorting(GetInt(Section, QString(CONFIG_LISTSORT).arg(Value->name()), 0), GetBool(Section, QString(CONFIG_LISTORDER).arg(Value->name()), false));

}

QString TConfig::GetAccel(QString Section, QAction *Value, QString DefAccel)
{
	QString key = GetStr(Section, QString(CONFIG_ACCEL).arg(Value->name()), DefAccel);
	if (key == "None")
		key = QString::null;
	return key;
}

void TConfig::GetStyleSheetItem(QString Section, QStyleSheetItem *Value, QColor DefaultColor, int DefStyle)
{
	Value->setColor(QColor(GetInt(Section, QString(CONFIG_TAGCOLOR).arg(Value->name()), DefaultColor.rgb())));
	int style = GetInt(Section, QString(CONFIG_TAGSTYLE).arg(Value->name()), DefStyle);
	Value->setFontWeight((style & STYLE_BOLD) ? QFont::Bold : QFont::Normal);
	Value->setFontItalic(style & STYLE_ITALIC);
	Value->setFontUnderline(style & STYLE_UNDERLINE);
}

void TConfig::GetToolBar(QWidget *Wnd, QToolBar *Value)
{
	int x = GetInt(QString::null, QString(CONFIG_TOOLBAR_POSX).arg(Wnd->name()), -1);
	int y = GetInt(QString::null, QString(CONFIG_TOOLBAR_POSY).arg(Wnd->name()), -1);
	while (x > ScreenRect.width())
		x -= 10;
	while (y > ScreenRect.height())
		y -= 10;
	if ((x != -1) && (y != -1))
	{
		Value->undock();
		Value->move(x, y);
		Value->setOrientation(GetBool(QString::null, QString(CONFIG_TOOLBAR_ORIENT).arg(Wnd->name()), true) ? Qt::Horizontal : Qt::Vertical);
		if (GetBool(QString::null, QString(CONFIG_TOOLBAR_DOCK).arg(Wnd->name()), true))
			Value->dock();
	}
	Value->setShown(GetBool(QString::null, QString(CONFIG_TOOLBAR_SHOW).arg(Wnd->name()), true));
}

void TConfig::SetInt(QString Section, QString Key, int Value)
{
	Settings->SetString(GetSection(Section), Key, QString("%1").arg(Value));
}

void TConfig::SetBool(QString Section, QString Key, bool Value)
{
	Settings->SetString(GetSection(Section), Key, Value ? "true" : "false");
}

void TConfig::SetStr(QString Section, QString Key, QString Value)
{
	Settings->SetString(GetSection(Section), Key, Value);
}

void TConfig::SetStrList(QString Section, QString ListName, QStringList *Value)
{
	int i, cnt = Value->count();

	SetInt(Section, QString(CONFIG_LIST_COUNTER).arg(ListName), cnt);
	for (i = 0; i < cnt; i++)
		SetStr(Section, QString(CONFIG_LIST_ITEM).arg(ListName).arg(i + 1), *Value->at(i));
}

void TConfig::SetDraftArea(QString Value)
{
	SetStr(QString::null, CONFIG_DRAFTAREA, Value);
}

void TConfig::SetPos(QWidget *Wnd)
{
	SetInt(QString::null, QString(CONFIG_WINDOWPOSX).arg(Wnd->name()), Wnd->pos().x());
	SetInt(QString::null, QString(CONFIG_WINDOWPOSY).arg(Wnd->name()), Wnd->pos().y());
}

void TConfig::SetSize(QWidget *Wnd)
{
	SetInt(QString::null, QString(CONFIG_WINDOWWIDTH).arg(Wnd->name()), Wnd->size().width());
	SetInt(QString::null, QString(CONFIG_WINDOWHEIGHT).arg(Wnd->name()), Wnd->size().height());
}

void TConfig::SetSplitter(QString Section, QSplitter *Value)
{
	int i = 0;

	QValueList<int> list = Value->sizes();
	for (QValueList<int>::Iterator it = list.begin(); it != list.end(); it++)
		SetInt(Section, QString("%1%2").arg(Value->name()).arg(i++), *it);
}

void TConfig::SetList(QString Section, QListView *Value)
{
	for (int i = 0; i < Value->columns(); i++)
		SetInt(Section, QString(CONFIG_LISTWIDTH).arg(Value->name()).arg(i), Value->columnWidth(i));
	SetInt(QString::null, QString(CONFIG_LISTSORT).arg(Value->name()), Value->sortColumn());
	SetBool(QString::null, QString(CONFIG_LISTORDER).arg(Value->name()), (Value->sortOrder() == Qt::Ascending) ? true : false);
}

void TConfig::SetAccel(QString Section, QAction *Value)
{
	QString key = Value->accel();
	SetStr(Section, QString(CONFIG_ACCEL).arg(Value->name()), key.isEmpty() ? "None" : key);
}

void TConfig::SetStyleSheetItem(QString Section, QStyleSheetItem *Value)
{
	SetInt(Section, QString(CONFIG_TAGCOLOR).arg(Value->name()), Value->color().rgb());
	int style = (Value->fontWeight() == QFont::Bold) ? STYLE_BOLD : 0;
	if (Value->fontItalic()) style |= STYLE_ITALIC;
	if (Value->fontUnderline()) style |= STYLE_UNDERLINE;
	SetInt(Section, QString(CONFIG_TAGSTYLE).arg(Value->name()), style);
}

void TConfig::SetToolBar(QWidget *Wnd, QToolBar *Value)
{
	SetBool(QString::null, QString(CONFIG_TOOLBAR_SHOW).arg(Wnd->name()), Value->isShown());
	SetBool(QString::null, QString(CONFIG_TOOLBAR_DOCK).arg(Wnd->name()), Value->place() == QDockWindow::InDock);
	SetBool(QString::null, QString(CONFIG_TOOLBAR_ORIENT).arg(Wnd->name()), Value->orientation() == Qt::Horizontal);
	SetInt(QString::null, QString(CONFIG_TOOLBAR_POSX).arg(Wnd->name()), Value->pos().x());
	SetInt(QString::null, QString(CONFIG_TOOLBAR_POSY).arg(Wnd->name()), Value->pos().y());
}

bool TConfig::IsDevelopMode()
{
	return GetBool(QString::null, "Develop", false);
}
