/***************************************************************************
 *   Copyright (C) 2005 by Alexander Shiyan                                *
 *   shc@users.sourceforge.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _TCONFIG_H_
#define _TCONFIG_H_

#include <qdir.h>
#include <qaction.h>
#include <qobject.h>
#include <qtoolbar.h>
#include <qlistview.h>
#include <qsplitter.h>
#include <qpopupmenu.h>
#include <qtextcodec.h>
#include <qstylesheet.h>
#include <qpixmapcache.h>
#include <qfontdatabase.h>
#include <qdesktopwidget.h>

#include "funchider.h"

class TConfig;

#include "utils.h"

#include "tarea.h"
#include "tbook.h"
#include "tfidoconfig.h"
#include "tinisettings.h"

#define CONFIG_SECTION_MAIN		"General"
#define CONFIG_SECTION_CONF		"Config"
#define CONFIG_SECTION_FIDO		"Fido"
#define CONFIG_SECTION_EDIT		"Editor"
#define CONFIG_SECTION_STYL		"StyleDlg"
#define CONFIG_SECTION_KEYB		"Keyboard"

#define CONFIG_WINDOWSTATE		"WindowState"

#define CONFIG_VIEWSTATUSBAR		"ViewStatusBar"
#define CONFIG_VIEWKLUDGES		"ViewKludges"
#define CONFIG_VIEWHIGHLIGHT		"ViewHighlight"
#define CONFIG_VIEWWORDWRAP		"ViewWordWrap"
#define CONFIG_VIEWORIGIN		"ViewOrigin"

#define CONFIG_FIDOCODEPAGE		"DefCodePage"
#define CONFIG_FIDODEFZONE		"DefZone"
#define CONFIG_FIDOCONFNAME		"ConfigName"

#define CONFIG_EXTBROWSER		"HtmlBrowser"

#define CONFIG_FONTNAME			"FontName"
#define CONFIG_FONTSIZE			"FontSize"
#define CONFIG_FONTCOLOR		"FontColor"
#define CONFIG_FONTBACK			"BackColor"
#define CONFIG_VIEWSTYLE		"HighlightStyle"

#define CONFIG_SEARCHAREA		"SearchArea"
#define CONFIG_SEARCHTEXT		"SearchText"
#define CONFIG_SEARCHCASE		"SearchCase"
#define CONFIG_SEARCHWHERE		"SearchWhere"
#define CONFIG_SEARCHREGEXP		"SearchRegExp"

#define CONFIG_REPLACELIST		"Replace"

#define CONFIG_LASTAREA			"LastArea"
#define CONFIG_LASTOPENSAVE		"LastDir"

#define STYLE_BOLD			1
#define STYLE_ITALIC			2
#define STYLE_UNDERLINE			4
#define STYLE_INVERSE			8

#define MSG_OPERATION_NEW		1
#define MSG_OPERATION_EDIT		2
#define MSG_OPERATION_REPLY		4
#define MSG_OPERATION_REPLYNET		8
#define MSG_OPERATION_FORWARD		16
#define MSG_OPERATION_UNSUBSCRIBE	32
//#define MSG_OPERATION_CONFIRM		64

#define TPL_MESSAGE_TPL			"message.tpl"
#define TPL_REPLY_TPL			"reply.tpl"
#define TPL_FORWARD_TPL			"forward.tpl"
#define TPL_UNSUBSCRIBE_TPL		"unsubscribe.tpl"
#define TPL_SAVETXT_TPL			"save_txt.tpl"
#define TPL_SAVEHTM_TPL			"save_htm.tpl"

#define TPL_ORIGIN_LST			"origin.lst"
#define TPL_KLUDGES_LST			"kludges.lst"

#define ADDRESSBOOK_NAME		"addressbook.xml"

#define REGEXP_ADDRESS			"^[\\d]{1,4}:[\\d]{1,4}/[\\d]{1,4}(\\.[\\d]{1,4}|\\b)(@[.\\S\\w]{1,15}|\\b)"
#define REGEXP_FROMTO			"^[\\S].{1,36}$"
#define REGEXP_EMAIL			"^[\\d\\w]{1,10}@[\\d\\w]{1,10}\\.[\\d\\w]{1,15}"
#define REGEXP_URL			"((ftp|http|https)://|www\\.)([\\d\\S\\w]{2,}\\.)+[\\d\\S\\w]{2,}($|(?=[\\s]))"

#define ERROR_INITFIDO_CONFIG		1
#define ERROR_INITFIDO_LOCK		2
#define ERROR_INITFIDO_SMAPI		3
#define ERROR_INITFIDO_SMAPI_OLD	4

#if defined(Q_WS_MAC)
#	define PLATFORM_NAME		"Mac"
#	define DEFAULT_VIEW_FONT	"???"
#	define DEFAULT_VIEW_FONTSIZE	???
#	define DEFAULT_EXT_BROWSER	???
#	define DEFAULT_EXT_SENDMAIL	???
#	ifdef ENABLE_BINRELOC
#		error "Configuration error!"
#	endif
#elif defined (Q_WS_WIN)
#	define PLATFORM_NAME		"Windows"
#	define DEFAULT_VIEW_FONT	"Lucida Console"
#	define DEFAULT_VIEW_FONTSIZE	10
#	define DEFAULT_EXT_BROWSER	"explorer %u"
#	define DEFAULT_EXT_SENDMAIL	"outlook"
#	if !defined(_open)
#		define _open		open
#		define _close		close
#	endif
#	ifdef ENABLE_BINRELOC
#		error "Configuration error!"
#	endif
#elif defined(Q_WS_X11)
#	define PLATFORM_NAME		"Linux"
#	define DEFAULT_VIEW_FONT	"console8x16"
#	define DEFAULT_VIEW_FONTSIZE	12
#	define DEFAULT_EXT_BROWSER	"mozilla %u"
#	define DEFAULT_EXT_SENDMAIL	"/usr/sbin/sendmail"
#	ifndef ENABLE_BINRELOC
#		error "Configuration error!"
#	endif
#else
#	error "Unable to determine target operating system!"
#endif

#define QFE_LOGO_NAME	"qfe_logo.xpm"
#define QFE_LOGO	QPixmap::fromMimeSource(QFE_LOGO_NAME)

typedef struct {
	void *InitProgress(uint);
	void *ProcProgress(uint);
	void *DoneProgress();
} TProgressCallBack;

class TConfig
{
public:
	TConfig(QDir*, QString = QString::null);
	~TConfig();

	ps_fidoconfig FidoConfig;

	QString ConfigPath;
	QString HomePath;

	QRect ScreenRect;
	QFontDatabase *FontDatabase;

	TBook *AddressBook;

	void FlushIni();

	void CheckTemplates();

	uint InitFido();
	void DoneFido();

	QString CharsetForMessages();

	QPixmap *FindPixmap(QString Name);

	void ChangeTextCodec(QString);
	QString toUTF(char*, uint = 0);
	QCString fromUTF(QString);

	int GetInt(QString, QString, int);
	bool GetBool(QString , QString, bool);
	QString GetStr(QString, QString, QString = QString::null);

	void GetStrList(QString, QString, QStringList*);

	QString GetDraftArea();

	void GetPos(QWidget*);
	void GetPos(QWidget*, int, int);
	void GetSize(QWidget*, int, int);
	void GetSplitter(QString, QSplitter*, const int* = NULL);
	void GetList(QString, QListView*, const int* = NULL);
	QString GetAccel(QString, QAction*, QString);
	void GetStyleSheetItem(QString, QStyleSheetItem*, QColor, int = 0);
	void GetToolBar(QWidget*, QToolBar *Value);

	void SetInt(QString, QString, int);
	void SetBool(QString, QString, bool);
	void SetStr(QString, QString, QString);

	void SetStrList(QString, QString, QStringList*);

	void SetDraftArea(QString);

	void SetPos(QWidget*);
	void SetSize(QWidget*);
	void SetSplitter(QString, QSplitter*);
	void SetList(QString, QListView*);
	void SetAccel(QString, QAction*);
	void SetStyleSheetItem(QString, QStyleSheetItem*);
	void SetToolBar(QWidget*, QToolBar *Value);

	bool IsDevelopMode();
private:
	struct _minf Smapi;
	int FidoConfLock;

	TIniSettings *Settings;
	QTextCodec *TextCodec;

	void RestoreFile(QString);
	QString GetSection(QString);
};

#endif // _TCONFIG_H_
