/***************************************************************************
 *   Copyright (C) 2005 by Alexander Shiyan                                *
 *   shc@users.sourceforge.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tmessage.h"

TMessage::TMessage(TArea *_Area, const uint _index, const bool DoCreate)
{
	Area = _Area;
	Index = _index;
	Handle = NULL;
	TextLen = 0;
	CtrlLen = 0;
	TextBuffer = NULL;
	CtrlBuffer = NULL;
	isReaded = false;
	if (!DoCreate)
	{
		if (Open(MOPEN_READ))
		{
			MsgReadMsg(Handle, &MsgInfo, 0, 0, NULL, 0, NULL);
			TextLen = MsgGetTextLen(Handle);
			CtrlLen = MsgGetCtrlLen(Handle);
			Close();
		}
	} else
		memset(&MsgInfo, '\0', sizeof(XMSG));
	MessageItem = NULL;
}

TMessage::~TMessage()
{
	if (TextBuffer != NULL)
		free(TextBuffer);
	if (CtrlBuffer != NULL)
		free(CtrlBuffer);
}

bool TMessage::Open(const uint Mode)
{
	if (Handle == NULL)
		if (Area->Open((Mode == MOPEN_READ) ? true : false))
			Handle = MsgOpenMsg(Area->Handle, Mode, Index);
	return (Handle != NULL);
}

void TMessage::Close()
{
	if (Handle != NULL)
	{
		MsgCloseMsg(Handle);
		Handle = NULL;
	}
}

uint TMessage::UID()
{
	return (Area->Index2UID(Index));
}

bool TMessage::ReadMessage()
{
	if (!isReaded)
	{
		if (Open(MOPEN_READ))
		{
			if (CtrlLen > 0)
			{
				CtrlBuffer = (byte*)malloc(CtrlLen + 1);
				MsgReadMsg(Handle, &MsgInfo, 0, 0, NULL, CtrlLen, CtrlBuffer);
				*(CtrlBuffer + CtrlLen) = '\0';
			}
			if (TextLen > 0)
			{
				TextBuffer = (byte*)malloc(TextLen + 1);
				MsgReadMsg(Handle, &MsgInfo, 0, TextLen, TextBuffer, 0, NULL);
				*(TextBuffer + TextLen) = '\0';
			}
			Close();
			isReaded = true;
		} else
			return false;
	}
	return true;
}

bool TMessage::WriteMessage(const bool DoCreate)
{
	bool ret = false;
	Close();
	if (Open(DoCreate ? MOPEN_CREATE : MOPEN_WRITE))
	{
		ret = MsgWriteMsg(Handle, 0, &MsgInfo, TextBuffer, TextLen, TextLen, CtrlLen, CtrlBuffer) == MERR_NONE;
		Close();
	}
	return ret;
}

bool TMessage::isFlagsSet(const dword flags)
{
	return ((MsgInfo.attr & flags) != 0);
}

bool TMessage::ToggleFlag(const dword flag, const bool state)
{
	dword s = MsgInfo.attr;
	if (state)
		s |= flag;
	else
		s &= ~flag;
	if (s != MsgInfo.attr)
	{
		ReadMessage();
		MsgInfo.attr = s;
		WriteMessage();
		return true;
	}
	return false;
}

void TMessage::MarkAsRead()
{
	if (ToggleFlag(MSGREAD, true))
	{
		if (Area->Unread)
			Area->Unread--;
		if (MessageItem != NULL)
			MessageItem->UpdateMessage();
	}
}
