/***************************************************************************
 *   Copyright (C) 2005 by Alexander Shiyan                                *
 *   shc@users.sourceforge.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tmessageitem.h"

extern TConfig *Config;

TMessageItem::TMessageItem(QListView *parent, TAreaItem *_AreaItem, TMessage *_Message) : QListViewItem(parent)
{
	setMultiLinesEnabled(false);
	AreaItem = _AreaItem;
	Message = _Message;
	Message->MessageItem = this;
	MarkState = 0;
	setText(0, QString::number(parent->childCount()));
	setText(1, QString::null);
	setText(2, Config->toUTF((char*)Message->MsgInfo.from));
	setText(3, Config->toUTF((char*)Message->MsgInfo.to));
	setText(4, Config->toUTF((char*)Message->MsgInfo.subj));
	setText(5, QString::number(Message->TextLen));
	setText(6, QDateTime(QDate(Message->MsgInfo.date_written.date.yr + 1980, Message->MsgInfo.date_written.date.mo, Message->MsgInfo.date_written.date.da), QTime(Message->MsgInfo.date_written.time.hh, Message->MsgInfo.date_written.time.mm, Message->MsgInfo.date_written.time.ss)).toString("hh:mm:ss dd.MM.yyyy"));
}

bool TMessageItem::isMarked()
{
    return (MarkState & MARK_MESSAGE_MARKED) != 0;
}

void TMessageItem::ToggleMarked()
{
	if (isMarked())
	{
		MarkState &= ~MARK_MESSAGE_MARKED;
		setPixmap(1, NULL);
	} else {
		MarkState |= MARK_MESSAGE_MARKED;
		setPixmap(1, *Config->FindPixmap("icon_mark.xpm"));
	}
	repaint();
}

void TMessageItem::StartTimer()
{
	if (!Message->isFlagsSet(MSGREAD))
		QTimer::singleShot(1000, this, SLOT(MarkMessageAsRead()));
	Message->Area->SetLastRead(Message->Index);
}

void TMessageItem::MarkMessageAsRead()
{
	if (isSelected())
	{
		Message->MarkAsRead();
		AreaItem->repaint();
		repaint();
	}
}

void TMessageItem::UpdateMessage()
{
	if (isVisible())
		repaint();
}

void TMessageItem::paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int alignment)
{
	if (column > 0)
	{
		QColorGroup _cg(cg);

		if (!Message->isFlagsSet(MSGREAD))
		{
			QFont f(p->font());
			f.setBold(true);
			p->setFont(f);
		}
		if (Message->Area->Type == AREA_TYPE_ECHOMAIL)
		{
			if (strcompare(text(2), Config->FidoConfig->sysop))
				_cg.setColor(QColorGroup::Text, Qt::red);
			else
				if (strcompare(text(3), Config->FidoConfig->sysop))
					_cg.setColor(QColorGroup::Text, Qt::red);
		}
		QListViewItem::paintCell(p, _cg, column, width, alignment | Qt::AlignTop);
	} else
		QListViewItem::paintCell(p, cg, column, width, Qt::AlignHCenter | Qt::AlignTop);
}

int TMessageItem::compare(QListViewItem *i, int col, bool ascending) const
{
	if ((col == 0) || (col == 5))
	{
		bool Test1 = false, Test2 = false;
		int Value1, Value2;

		Value1 = key(col, ascending).toInt(&Test1);
		Value2 = i->key(col, ascending).toInt(&Test2);
		if (Test2 && Test2)
			return ((Value1 > Value2) ? -1 : ((Value1 < Value2) ? 1 : 0));
	} else if (col == 6)
	{
		QDateTime dt1(QDate(Message->MsgInfo.date_written.date.yr + 1980, Message->MsgInfo.date_written.date.mo, Message->MsgInfo.date_written.date.da), QTime(Message->MsgInfo.date_written.time.hh, Message->MsgInfo.date_written.time.mm, Message->MsgInfo.date_written.time.ss));
		QDateTime dt2(QDate(((TMessageItem*)i)->Message->MsgInfo.date_written.date.yr + 1980, ((TMessageItem*)i)->Message->MsgInfo.date_written.date.mo, ((TMessageItem*)i)->Message->MsgInfo.date_written.date.da), QTime(((TMessageItem*)i)->Message->MsgInfo.date_written.time.hh, ((TMessageItem*)i)->Message->MsgInfo.date_written.time.mm, ((TMessageItem*)i)->Message->MsgInfo.date_written.time.ss));
		return ((dt1 > dt2) ? -1 : ((dt1 < dt2) ? 1 : 0));
	}

	return QListViewItem::compare(i, col, ascending);
}
