/***************************************************************************
 *   Copyright (C) 2005 by Alexander Shiyan                                *
 *   shc@users.sourceforge.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tnodes.h"

extern TConfig *Config;

TNode::TNode(word *zone, word *region, word *node, word *point, uint _type, char *info)
{
	addr.zone = *zone;
	addr.net = *region;
	addr.node = *node;
	addr.point = *point;

	type = _type;
	line = (info != NULL) ? info : QString::null;
};

TNodes::TNodes()
{
	ListView = NULL;
	setAutoDelete(true);
}

void TNodes::AssignListView(QListView *lv)
{
	TNode *Node;
	TNodeItem *ZoneRoot = NULL, *HostRoot = NULL, *NodeRoot = NULL, *Point = NULL;

	ListView = lv;
	if (ListView != NULL)
	{
		ListView->clear();
		for (uint i = 0; i < count(); i++)
		{
			Node = at(i);
			switch (Node->type)
			{
				case RECORD_TYPE_ZONE:
					ZoneRoot = new TNodeItem(ListView, ZoneRoot, Node);
					HostRoot = ZoneRoot;
					NodeRoot = ZoneRoot;
					Point = NULL;
					break;
				case RECORD_TYPE_REGION:
				case RECORD_TYPE_HOST:
					if (ZoneRoot != NULL)
					{
						if (Node->addr.zone != ZoneRoot->Node->addr.zone)
						{
							HostRoot = new TNodeItem(ListView, ZoneRoot, Node);
							ZoneRoot = NULL;
						} else
							HostRoot = new TNodeItem(ZoneRoot, HostRoot, Node);
					} else
						HostRoot = new TNodeItem(ListView, HostRoot, Node);
					NodeRoot = HostRoot;
					Point = NULL;
					break;
				case RECORD_TYPE_HUB:
				case RECORD_TYPE_PVT:
				case RECORD_TYPE_HOLD:
				case RECORD_TYPE_DOWN:
				case RECORD_TYPE_NODE:
					if (HostRoot != NULL)
					{
						if (Node->addr.zone != HostRoot->Node->addr.zone)
						{
							if (ZoneRoot != NULL)
							{
								if (Node->addr.net != HostRoot->Node->addr.net)
									NodeRoot = new TNodeItem(ZoneRoot, NodeRoot, Node);
								else
									NodeRoot = new TNodeItem(HostRoot, NodeRoot, Node);
								HostRoot = NULL;
							} else
								NodeRoot = new TNodeItem(HostRoot, NodeRoot, Node);
							ZoneRoot = NULL;
						} else
							NodeRoot = new TNodeItem(HostRoot, NodeRoot, Node);
					} else if (ZoneRoot != NULL)
						NodeRoot = new TNodeItem(ZoneRoot, NodeRoot, Node);
					else
						NodeRoot = new TNodeItem(ListView, NodeRoot, Node);
					Point = NULL;
					break;
				case RECORD_TYPE_POINT:
					if (NodeRoot != NULL)
					{
						if ((Node->addr.zone == NodeRoot->Node->addr.zone) && (Node->addr.net == NodeRoot->Node->addr.net) && (Node->addr.node == NodeRoot->Node->addr.node))
							Point = new TNodeItem(NodeRoot, Point, Node);
						else
							Point = NULL;

					}
					break;
				default:
					break;
			}
		}
	}
}

int TNodes::Find(word zone, word region, word node, word point)
{
	TNode *It = new TNode(&zone, &region, &node, &point, 0, NULL);

	int ret = find(It);

	delete It;

	return ret;
}

void TNodes::AddNode(word *zone, word *region, word *node, word *point, uint _type, char *info)
{
	TNode *It = new TNode(zone, region, node, point, _type, NULL);

	int found = find(It);
	if (found > -1)
	{
		if ((info != NULL) && at(found)->line.isEmpty())
			at(found)->line = info;
		delete It;
	} else {
		if (info != NULL)
			It->line = info;
		append(It);
	}
}

bool TNodes::ParseString(char *str, word *num, char *strout)
{
	char *tmp = str + 1;

	if (*str == ',')
	{
		while((*tmp >= '0') && (*tmp <= '9'))
			tmp++;
		*tmp = '\0';
		*num = (word)atol(str + 1);
		strcpy(strout, tmp + 1);
		return true;
	}
	return false;
}

void TNodes::ProcessLine(int format, char *line, word *zone, word *region, word *node)
{
	char info[NDL_MAX_LINE_LENGTH];
	word n, point = 0;

	if (*line == ',')
	{
		switch (format)
		{
			case points4d:
				break;
			case points24:
				if ((*zone > 0) && (*region > 0) && (*node > 0) && ParseString(line, &n, &info[0]))
					if (Find(*zone, *region, *node) > -1)
						AddNode(zone, region, node, &n, RECORD_TYPE_POINT, &info[0]);
				break;
			default:
				if ((*zone > 0) && (*region > 0) && ParseString(line, &n, &info[0]))
				{
					*node = n;
					AddNode(zone, region, node, &point, RECORD_TYPE_NODE, &info[0]);
				}
				break;
		}
	} else {
		switch (format)
		{
			case points4d:
				if ((*zone > 0) && (*region > 0) && (*node > 0) && !qstrnicmp(line, "point", 5))
				{
					if (ParseString(line + 5, &n, &info[0]))
						if (n > 0)
							AddNode(zone, region, node, &n, RECORD_TYPE_POINT, &info[0]);
				} else if (!qstrnicmp(line, "boss", 4))
				{
					*zone = 0;
					*region = 0;
					*node = 0;
					if (sscanf(line + 4, ",%s", &info[0]) == 1)
					{
						hs_addr addr;
						string2addr(&info[0], &addr);
						if (addr.zone > 0)
						{
							*zone = addr.zone;
							*region = addr.net;
							*node = addr.node;
						} else {
							*zone = 0;
							*region = 0;
							*node = 0;
						}
					}
				} else {
					*zone = 0;
					*region = 0;
					*node = 0;
				}
				break;
			case points24:
				*node = 0;
				if ((*zone > 0) && (*region > 0) && !qstrnicmp(line, "host", 4))
				{
					if (ParseString(line + 4, &n, &info[0]))
					{
						*node = n;
						AddNode(zone, region, node, &point, RECORD_TYPE_NODE, &info[0]);
					}
				}
				break;
			default:
				*node = 0;
				if ((*zone > 0) && (*region > 0) && !qstrnicmp(line, "hub", 3))
				{
					if (ParseString(line + 3, &n, &info[0]))
					{
						*node = n;
						AddNode(zone, region, node, &point, RECORD_TYPE_HUB, &info[0]);
					}
				} else if ((*zone > 0) && (*region > 0) && !qstrnicmp(line, "pvt", 3))
				{
					if (ParseString(line + 3, &n, &info[0]))
					{
						*node = n;
						AddNode(zone, region, node, &point, RECORD_TYPE_PVT, &info[0]);
					}
				} else if ((*zone > 0) && (*region > 0) && !qstrnicmp(line, "hold", 4))
				{
					if (ParseString(line + 4, &n, &info[0]))
					{
						*node = n;
						AddNode(zone, region, node, &point, RECORD_TYPE_HOLD, &info[0]);
					}
				} else if ((*zone > 0) && (*region > 0) && !qstrnicmp(line, "down", 4))
				{
					if (ParseString(line + 4, &n, &info[0]))
					{
						*node = n;
						AddNode(zone, region, node, &point, RECORD_TYPE_DOWN, &info[0]);
					}
				} else if (!qstrnicmp(line, "zone", 4))
				{
					*region = 0;
					if (ParseString(line + 4, &n, &info[0]))
					{
						*zone = n;
						AddNode(zone, region, node, &point, RECORD_TYPE_ZONE, &info[0]);
					}
				} else if ((*zone > 0) && !qstrnicmp(line, "region", 6))
				{
					if (ParseString(line + 6, &n, &info[0]))
					{
						*region = n;
						AddNode(zone, region, node, &point, RECORD_TYPE_REGION, &info[0]);
					}
				} else if ((*zone > 0) && !qstrnicmp(line, "host", 4))
				{
					if (ParseString(line + 4, &n, &info[0]))
					{
						*region = n;
						AddNode(zone, region, node, &point, RECORD_TYPE_HOST, &info[0]);
					}
				}
				break;
		}
	}
}

bool TNodes::ProcessList(char *fn, nodelist *ndl)
{
	int linelen;
	word region = 0, node = 0;
	char line[NDL_MAX_LINE_LENGTH + 1];
	word zone = (ndl->defaultZone > 0) ? ndl->defaultZone : Config->GetInt(CONFIG_SECTION_FIDO, CONFIG_FIDODEFZONE, 2);
	QFile ndf(fn);

	if (FileOpenFunc(&ndf, IO_ReadOnly))
	{
		while (!ndf.atEnd())
		{
			linelen = ndf.readLine(&line[0], NDL_MAX_LINE_LENGTH);
			while ((linelen > 0) && (line[linelen - 1] < 0x21))
				linelen--;
			if ((linelen >= 6) && (line[0] != ';'))
			{
				line[linelen] = '\0';
				ProcessLine(ndl->format, &line[0], &zone, &region, &node);
			} else if (linelen <= 0)
				break;
		}
		FileCloseFunc(&ndf);
		return true;
	}

	return false;
}

uint TNodes::TestCount()
{
	uint i, j, ret = 0;

	for (i = 0; i < Config->FidoConfig->nodelistCount; i++)
	{
		nodelist ndl = Config->FidoConfig->nodelists[i];
		QDir ndldir(Config->FidoConfig->nodelistDir, QString(ndl.nodelistName) + ".???", QDir::Time | QDir::Reversed, QDir::Files | QDir::Readable);

		j = 0;
	        while (j < ndldir.count())
			if (testexists(Config->FidoConfig->nodelistDir + ndldir.entryList()[j]))
			{
				ret++;
				break;
			} else
				j++;
	}

	return ret;
}

uint TNodes::Compile()
{
	uint i, j, ret = 0;

	for (i = 0; i < Config->FidoConfig->nodelistCount; i++)
	{
		nodelist ndl = Config->FidoConfig->nodelists[i];
		QDir ndldir(Config->FidoConfig->nodelistDir, QString(ndl.nodelistName) + ".???", QDir::Time | QDir::Reversed, QDir::Files | QDir::Readable);
		j = 0;
	        while (j < ndldir.count())
		{
			QString ndlfile = Config->FidoConfig->nodelistDir + ndldir.entryList()[j];
			if (testexists(ndlfile))
			{
				if (ProcessList((char*)ndlfile.ascii(), &ndl))
				{
					ret++;
					break;
				}
			}
			j++;
			qApp->processEvents();
		}
	}

	sort();

	return ret;
}

int TNodes::compareItems(QPtrCollection::Item It1, QPtrCollection::Item It2)
{
	NETADDR *addr1 = &((TNode*)It1)->addr;
	NETADDR *addr2 = &((TNode*)It2)->addr;

	if (addr1->zone > addr2->zone)
		return 1;
	if (addr1->zone < addr2->zone)
		return -1;

	if (addr1->net > addr2->net)
		return 1;
	if (addr1->net < addr2->net)
		return -1;

	if (addr1->node > addr2->node)
		return 1;
	if (addr1->node < addr2->node)
		return -1;

	if (addr1->point > addr2->point)
		return 1;
	if (addr1->point < addr2->point)
		return -1;

	return 0;
}
