/***************************************************************************
 *   Copyright (C) 2005 by Alexander Shiyan                                *
 *   shc@users.sourceforge.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

extern TConfig *Config;

void TAddressBook::init()
{
	FirstTimeInit = false;

        setIcon(*Config->FindPixmap("icon_addressbook.xpm"));

	BtnOk->setIconSet(QIconSet(*Config->FindPixmap("icon_ok.xpm")));
	BtnCancel->setIconSet(QIconSet(*Config->FindPixmap("icon_cancel.xpm")));

	BtnAdd->setIconSet(QIconSet(*Config->FindPixmap("icon_additem.xpm")));
	BtnDel->setIconSet(QIconSet(*Config->FindPixmap("icon_delitem.xpm")));

	BtnPhoto->setIconSet(QIconSet(*Config->FindPixmap("icon_photo.xpm")));

	AddressList->setColumnText(0, QIconSet(*Config->FindPixmap("icon_name.xpm")), QObject::tr("User"));

	Config->GetPos(this);
	SelectMode = false;
	Selected = NULL;

	UserEdit->setValidator(new QRegExpValidator(QRegExp(REGEXP_FROMTO), this));
	EMailEdit->setValidator(new QRegExpValidator(QRegExp(REGEXP_EMAIL), this));
	AddressEdit->setValidator(new QRegExpValidator(QRegExp(REGEXP_ADDRESS), this));
}

void TAddressBook::SetUp(bool DoSelect)
{
	SelectMode = DoSelect;
}

void TAddressBook::showEvent(QShowEvent *)
{
	if (FirstTimeInit)
		return;
	BtnOk->setEnabled(not SelectMode);
	for (uint i = 0; i < Config->AddressBook->count(); i++)
		new TBookViewItem(AddressList, Config->AddressBook->at(i));

	BtnAdd->setEnabled(not SelectMode);
	UserEdit->setReadOnly(SelectMode);
	EMailEdit->setReadOnly(SelectMode);
	AddressEdit->setReadOnly(SelectMode);
	CommentEdit->setReadOnly(SelectMode);
}

void TAddressBook::closeEvent(QCloseEvent *e)
{
	Config->SetPos(this);
	e->accept();
}

void TAddressBook::ActionAdd()
{
	TBookItem *Item = Config->AddressBook->Add();
	Item->Name = "SysOp";
	Item->Comment = "Added " + QDateTime::currentDateTime().toString();
	AddressList->setSelected(new TBookViewItem(AddressList, Item), true);
}

void TAddressBook::CleanData()
{
	BtnOk->setEnabled(not SelectMode);

	UserEdit->setEnabled(false);
	EMailEdit->setEnabled(false);
	AddressEdit->setEnabled(false);
	CommentEdit->setEnabled(false);

	UserEdit->setText(QString::null);
	EMailEdit->setText(QString::null);
	AddressEdit->setText(QString::null);
	CommentEdit->setText(QString::null);

	Selected = NULL;
}

void TAddressBook::ActionRemove()
{
	if (Selected == NULL)
		return;
	int idx = Config->AddressBook->Index(Selected);
	if (idx > -1)
	{
		CleanData();

		Config->AddressBook->Delete(idx);
		AddressList->removeItem(AddressList->selectedItem());
		BtnDel->setEnabled(Config->AddressBook->count() > 0);
		if (SelectMode)
			BtnOk->setEnabled(Config->AddressBook->count() > 0);
	}
}

void TAddressBook::ListSelectionChanged(QListViewItem *it)
{
	CleanData();

	if (it != NULL)
	{
		BtnOk->setEnabled(true);
		BtnDel->setEnabled(not SelectMode);

		Selected = ((TBookViewItem*)it)->BookItem;

		UserEdit->setText(Selected->Name);
		EMailEdit->setText(Selected->EMail);
		AddressEdit->setText(Selected->Address);
		CommentEdit->setText(Selected->Comment);

		UserEdit->setEnabled(true);
		EMailEdit->setEnabled(true);
		AddressEdit->setEnabled(true);
		CommentEdit->setEnabled(true);
	}
}

void TAddressBook::UserChanged(const QString&)
{
	CommentChanged();
}

void TAddressBook::CommentChanged()
{
	if (UserEdit->isEnabled() && (Selected != NULL))
	{
		Selected->Name = UserEdit->text().stripWhiteSpace();
		Selected->EMail = EMailEdit->text().stripWhiteSpace();
		Selected->Address = AddressEdit->text().stripWhiteSpace();
		Selected->Comment = CommentEdit->text().stripWhiteSpace();

		int pos = 0;
		UserEdit->setPaletteForegroundColor(UserEdit->validator()->validate((QString&)Selected->Name, pos) == QValidator::Acceptable ? Qt::black : Qt::red);
		pos = 0;
		EMailEdit->setPaletteForegroundColor(EMailEdit->validator()->validate((QString&)Selected->EMail, pos) == QValidator::Acceptable ? Qt::black : Qt::red);
		pos = 0;
		AddressEdit->setPaletteForegroundColor(AddressEdit->validator()->validate((QString&)Selected->Address, pos) == QValidator::Acceptable ? Qt::black : Qt::red);

		AddressList->selectedItem()->repaint();
	}
}

void TAddressBook::ListDoubleClicked(QListViewItem*)
{
	if (SelectMode)
		accept();
}
