/***************************************************************************
 *   Copyright (C) 2005 by Alexander Shiyan                                *
 *   shc@users.sourceforge.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

extern TConfig *Config;
extern TAreas *Areas;

#define MESS_NOTAREA		QObject::tr("Can't find Area.")
#define MESS_NOTMESSAGE		QObject::tr("Can't find Message.")

#define DEFMESSAGEFLASG		MSGLOCAL

void TEditWindow::init()
{
	FirstTimeInit = false;

	setIcon(*Config->FindPixmap("icon_edit.xpm"));

	BtnAttributes->setIconSet(QIconSet(*Config->FindPixmap("icon_flag.xpm")));
	BtnAddrBook->setIconSet(QIconSet(*Config->FindPixmap("icon_addressbook.xpm")));
	BtnNodeList->setIconSet(QIconSet(*Config->FindPixmap("icon_browser.xpm")));

	EditActions = new TActions();

	EditActions->AddAction(MessageMenu, EditMenuBar->text(1), SaveMessageAction, "icon_sent.xpm", QObject::tr("Save Message to MessageBase"), QString::null, "Ctrl+S");
	EditActions->AddAction(MessageMenu, EditMenuBar->text(1), SaveDraftAction, "icon_draft.xpm", QObject::tr("Save Message to Draft Folder"), QObject::tr("You must continue editing this message later."), "Ctrl+D");
	EditActions->AddAction(MessageMenu, EditMenuBar->text(1), QuitAction, "icon_exit.xpm", QObject::tr("Exit from Editor without Save"), QString::null, "Ctrl+Q");

	EditActions->AddAction(EditMenu, EditMenuBar->text(2), UndoAction, "icon_undo.xpm", QObject::tr("Undo Last Operation"), QString::null, "Ctrl+Z");
	EditActions->AddAction(EditMenu, EditMenuBar->text(2), RedoAction, "icon_redo.xpm", QObject::tr("Redo Last Operation"), QString::null, "Ctrl+Shift+Z");
	EditActions->AddAction(EditMenu, EditMenuBar->text(2), CopyAction, "icon_copy.xpm", QObject::tr("Copy selected text to Clipboard"), QString::null, "Ctrl+C");
	EditActions->AddAction(EditMenu, EditMenuBar->text(2), CutAction, "icon_cut.xpm", QObject::tr("Cut selected text to Clipboard"), QString::null, "Ctrl+X");
	EditActions->AddAction(EditMenu, EditMenuBar->text(2), DelAction, "icon_delete.xpm", QObject::tr("Delete selected text"), QString::null, "Ctrl+D");
	EditActions->AddAction(EditMenu, EditMenuBar->text(2), PasteAction, "icon_paste.xpm", QObject::tr("Paste text from Clipboard"), QString::null, "Ctrl+V");
	EditActions->AddAction(EditMenu, EditMenuBar->text(2), InsertAction, "icon_insert.xpm", QObject::tr("Insert Text file into Current Message"));
	EditActions->AddAction(EditMenu, EditMenuBar->text(2), AttachAction, "icon_attach.xpm", QObject::tr("Insert File into Current Message in UU-Format"), QString::null, QString::null, true);

	EditActions->AddAction(ViewMenu, EditMenuBar->text(3), ViewToolBarAction, QString::null, QObject::tr("Show/Hide ToolBar"));
	EditActions->AddAction(ViewMenu, EditMenuBar->text(3), ViewStatusBarAction, QString::null, QObject::tr("Show/Hide StatusBar"));
	EditActions->AddAction(ViewMenu, EditMenuBar->text(3), ViewOriginAction, QString::null, QObject::tr("Show/Hide Origin Line"));

	EditActions->AddAction(ToolMenu, EditMenuBar->text(4), SpellCheckAction, "icon_spell.xpm", QObject::tr("Check Message for Spelling Errors"), QString::null, QString::null, true);

	EditActions->AddAction(HelpMenu, EditMenuBar->text(5), HelpAction, "icon_help.xpm", QObject::tr("View Program Help"), QString::null, "F1", true);
	EditActions->AddAction(HelpMenu, EditMenuBar->text(5), WhatsThisAction, "icon_whatsthis.xpm", QObject::tr("Enter in \"What is This?\"-Mode"), QString::null, "Shift+F1");

	WhatsThisAction->addTo(EditToolBar);
	EditToolBar->setCloseMode(QDockWindow::Undocked);

	Config->GetPos(this);
	Config->GetSize(this, size().width(), size().height());
	ToNameCombo->setValidator(new QRegExpValidator(QRegExp(REGEXP_FROMTO), this));
	ToAddrEdit->setValidator(new QRegExpValidator(QRegExp(REGEXP_ADDRESS), this));
	FromNameCombo->setValidator(new QRegExpValidator(QRegExp(REGEXP_FROMTO), this));

	Config->GetToolBar(this, EditToolBar);
	ViewToolBarAction->setOn(EditToolBar->isShown());

	ViewStatusBarAction->setOn(Config->GetBool(CONFIG_SECTION_EDIT, CONFIG_VIEWSTATUSBAR, true));
	ActionViewStatusBar();

	ViewOriginAction->setOn(Config->GetBool(CONFIG_SECTION_EDIT, CONFIG_VIEWORIGIN, true));
	ActionViewOrigin();

	MessageEdit->setFont(QFont(Config->GetStr(CONFIG_SECTION_EDIT, CONFIG_FONTNAME, DEFAULT_VIEW_FONT), Config->GetInt(CONFIG_SECTION_EDIT, CONFIG_FONTSIZE, DEFAULT_VIEW_FONTSIZE)));
	MessageEdit->setPaletteBackgroundColor(QColor(Config->GetInt(CONFIG_SECTION_EDIT, CONFIG_FONTBACK, QColor(0, 0, 0).rgb())));
	MessageEdit->setPaletteForegroundColor(QColor(Config->GetInt(CONFIG_SECTION_EDIT, CONFIG_FONTCOLOR, QColor(220, 220, 220).rgb())));

	OriginLabel->setText("Origin:");

	MessageEdit->installEventFilter(this);

	OrigArea = QString::null;
	DestArea = QString::null;
	UniqueID = 0;
	SetupDone = false;
	Result = false;
}

bool TEditWindow::SetUp(TArea *Area, TMessage *Message, uint Operation, void *addval)
{
	uint i;
	QListBoxItem *tmpit;
	int textcnt = 0;
	char *reply = NULL;
	QString tmp, tmpaddr;
	QPoint pos;

	MsgOperation = Operation;

	Kludges.clear();
	MessageEdit->clear();

	ToNameCombo->clear();

	FromAddrCombo->clear();
	FromAddrCombo->insertItem(Area->Aka);
	hs_addr *_addr = Config->FidoConfig->addr;
	for (i = 0; i < Config->FidoConfig->addrCount; i++)
	{
		tmp = addr2str2(*_addr++);
		if (!strcompare(tmp, FromAddrCombo->text(0)))
			FromAddrCombo->insertItem(tmp);
	}
	FromNameCombo->clear();
	FromNameCombo->insertItem(Config->FidoConfig->sysop);

	OriginCombo->clear();

	switch (MsgOperation)
	{
		case MSG_OPERATION_NEW:
		{
			OrigArea = Area->Name;
			DestArea = Area->Name;

			MessFlags = DEFMESSAGEFLASG;

			if (Area->Type == AREA_TYPE_ECHOMAIL)
			{
				ToNameCombo->insertItem("All", 0);
				AllowFlags = MSGKILL | MSGLOCAL;

			} else {
				ToNameCombo->insertItem("", 0);
				ToAddrEdit->setEnabled(true);
				MessFlags |= MSGPRIVATE;
				AllowFlags = MSGPRIVATE | MSGCRASH | MSGFILE | MSGKILL | MSGLOCAL | MSGHOLD | MSGXX2 | MSGFRQ | MSGRRQ | MSGARQ | MSGURQ | MSGIMM;
			}

			LoadTemplate(TPL_MESSAGE_TPL, &pos);
			ParseTemplate(Area, NULL, false);
			break;
		}
		case MSG_OPERATION_EDIT:
		{
			OrigArea = Message->Area->Name;
			if (strcompare(Area->Name, Config->GetDraftArea()))
			{
				tmp = (char*)GetCtrlToken(Message->CtrlBuffer, (byte*)"AREA: ");
				if (tmp.isEmpty())
				{
					errormessage(this, MESS_NOTAREA);
					return false;
				}
				DestArea = tmp.ascii() + 6;
			} else
				DestArea = Area->Name;

			MessFlags = Message->MsgInfo.attr;

			i = Areas->Find(DestArea);
			if (i == 0)
			{
				errormessage(this, MESS_NOTAREA);
				return false;
			}
			TArea *DstArea = Areas->at(i - 1);
			if (DstArea->Type == AREA_TYPE_ECHOMAIL)
			{
				AllowFlags = MSGKILL | MSGLOCAL;
			} else {
				ToAddrEdit->setEnabled(true);
				ToAddrEdit->setText(addr2str1(Message->MsgInfo.dest, Message->CtrlBuffer));
				AllowFlags = MSGPRIVATE | MSGCRASH | MSGFILE | MSGKILL | MSGLOCAL | MSGHOLD | MSGXX2 | MSGFRQ | MSGRRQ | MSGARQ | MSGURQ | MSGIMM;
			}

			tmp = addr2str1(Message->MsgInfo.orig);
			tmpit = FromAddrCombo->listBox()->findItem(tmp);
			if (tmpit != NULL)
				FromAddrCombo->setCurrentItem(FromAddrCombo->listBox()->index(tmpit));
			tmp = (char*)Message->MsgInfo.from;
			tmpit = FromNameCombo->listBox()->findItem(tmp);
			if (tmpit == NULL)
			{
				FromNameCombo->insertItem(tmp, 0);
				FromNameCombo->setCurrentItem(0);
			} else
				FromNameCombo->setCurrentItem(FromNameCombo->listBox()->index(tmpit));

			ToNameCombo->insertItem((char*)Message->MsgInfo.to, 0);
			SubjEdit->setText(Config->toUTF((char*)Message->MsgInfo.subj));

			if (DstArea->Type == AREA_TYPE_NETMAIL)
			{
				reply = (char*)GetCtrlToken(Message->CtrlBuffer, (byte *)"INTL");
				if (reply != NULL)
				{
					Kludges.append("INTL" + QString(reply + 4));
					nfree(reply);
				}
			}
			reply = (char*)GetCtrlToken(Message->CtrlBuffer, (byte *)"REPLY:");
			if (reply != NULL)
			{
				Kludges.append(reply);
				nfree(reply);
			}

			UniqueID = Message->UID();

			MessageEdit->append("@text@");
			textcnt = LoadText(Message);
			break;
		}
		case MSG_OPERATION_REPLY:
		{
			OrigArea = Message->Area->Name;
			DestArea = Area->Name;

			MessFlags = DEFMESSAGEFLASG;

			ToNameCombo->insertItem((char*)Message->MsgInfo.from, 0);
			if (Area->Type == AREA_TYPE_NETMAIL)
			{
				ToAddrEdit->setText(addr2str1(Message->MsgInfo.orig, Message->CtrlBuffer));
				ToAddrEdit->setEnabled(true);
				MessFlags |= MSGPRIVATE;
				AllowFlags = MSGPRIVATE | MSGCRASH | MSGFILE | MSGKILL | MSGLOCAL | MSGHOLD | MSGXX2 | MSGFRQ | MSGRRQ | MSGARQ | MSGURQ | MSGIMM;
			} else
				AllowFlags = MSGKILL | MSGLOCAL;

			tmp = Config->toUTF((char*)Message->MsgInfo.subj);
			if (!tmp.startsWith("Re", false))
				tmp = "Re: " + tmp;
			SubjEdit->setText(tmp);

			reply = (char*)GetCtrlToken(Message->CtrlBuffer, (byte *)"MSGID:");
			if (reply != NULL)
			{
				Kludges.append("REPLY" + QString(reply + 5));
				nfree(reply);
			}

			LoadTemplate(TPL_REPLY_TPL, &pos);
			ParseTemplate(Area, Message, false);
			textcnt = LoadText(Message);
			break;
		}
		case MSG_OPERATION_REPLYNET:
		{
			OrigArea = Message->Area->Name;
			DestArea = Area->Name;

			MessFlags = DEFMESSAGEFLASG | MSGPRIVATE;
			AllowFlags = MSGPRIVATE | MSGCRASH | MSGFILE | MSGKILL | MSGLOCAL | MSGHOLD | MSGXX2 | MSGFRQ | MSGRRQ | MSGARQ | MSGURQ | MSGIMM;

			ToAddrEdit->setText(addr2str1(Message->MsgInfo.orig, Message->CtrlBuffer));
			ToAddrEdit->setEnabled(true);
			ToNameCombo->insertItem((char*)Message->MsgInfo.from, 0);

			tmp = Config->toUTF((char*)Message->MsgInfo.subj);
			if (!tmp.startsWith("Re", false))
				tmp = "Re: " + tmp;
			SubjEdit->setText(tmp);

			reply = (char*)GetCtrlToken(Message->CtrlBuffer, (byte *)"MSGID:");
			if (reply != NULL)
			{
				Kludges.append("REPLY" + QString(reply + 5));
				nfree(reply);
			}

			LoadTemplate(TPL_REPLY_TPL, &pos);
			ParseTemplate(Area, Message, false);
			textcnt = LoadText(Message);
			break;
		}
		case MSG_OPERATION_FORWARD:
		{
			OrigArea = Message->Area->Name;
			DestArea = Area->Name;

			MessFlags = DEFMESSAGEFLASG;

			if (Area->Type == AREA_TYPE_ECHOMAIL)
			{
				ToNameCombo->insertItem("All", 0);
				AllowFlags = MSGKILL | MSGLOCAL;
			} else {
				ToNameCombo->insertItem("", 0);
				ToAddrEdit->setEnabled(true);
				MessFlags |= MSGPRIVATE;
				AllowFlags = MSGPRIVATE | MSGCRASH | MSGFILE | MSGKILL | MSGLOCAL | MSGHOLD | MSGXX2 | MSGFRQ | MSGRRQ | MSGARQ | MSGURQ | MSGIMM;
			}

			tmp = Config->toUTF((char*)Message->MsgInfo.subj);
			if (!tmp.startsWith("Fwd", false))
				tmp = "Fwd: " + tmp;
			SubjEdit->setText(tmp);

			LoadTemplate(TPL_FORWARD_TPL, &pos);
			ParseTemplate(Area, Message, false);
			textcnt = LoadText(Message);
			break;
		}
		case MSG_OPERATION_UNSUBSCRIBE:
		{
			OrigArea = ((TArea*)addval)->Name;
			DestArea = Area->Name;

			MessFlags = DEFMESSAGEFLASG | MSGPRIVATE| MSGKILL | MSGXX2;
			AllowFlags = MSGPRIVATE | MSGCRASH | MSGKILL | MSGLOCAL | MSGHOLD | MSGXX2 | MSGIMM;

			ToAddrEdit->setEnabled(true);
			ToAddrEdit->setText(((TArea*)addval)->Uplink);
			SubjEdit->setText(((TArea*)addval)->UplinkPwd);

			ToNameCombo->insertItem(((TArea*)addval)->UplinkRbt, 0);

			LoadTemplate(TPL_UNSUBSCRIBE_TPL, &pos);
			ParseTemplate(Area, NULL, false);
			tmp = MessageEdit->text();
			tmp.replace("@origarea@", OrigArea);
			MessageEdit->setText(tmp);
			break;
		}
		default:
			return false;
	}

	setCaption(QString(QObject::tr("Message Editor") + " [" + QObject::tr("Area") + ": %1]").arg(DestArea));

	if (pos.isNull())
		pos.setY(MessageEdit->paragraphs() - 1);
	else
		pos.setY(pos.y() + textcnt);
	MessageEdit->setCursorPosition(pos.y(), pos.x());

	ClearModified();

	if ((MsgOperation != MSG_OPERATION_UNSUBSCRIBE) && ((MsgOperation != MSG_OPERATION_EDIT) || strcompare(Area->Name, Config->GetDraftArea())))
	{
		tmp = Config->ConfigPath + TPL_ORIGIN_LST + "." + DestArea;
		if (!testexists(tmp))
			tmp = Config->ConfigPath + TPL_ORIGIN_LST;
		if (testexists(tmp))
		{
			QFile origins(tmp);
			if (FileOpenFunc(&origins, IO_ReadOnly))
			{
				OriginCombo->insertItem("");

				QTextStream stream(&origins);
				while (!stream.atEnd())
				{
					tmp = stream.readLine().stripWhiteSpace();
					if (!tmp.isEmpty())
						OriginCombo->insertItem(tmp);
				}
				FileCloseFunc(&origins);

				if (OriginCombo->listBox()->count() > 1)
					OriginCombo->setCurrentItem(1);
			}
		}
	} else {
		ViewOriginAction->setEnabled(false);
		ViewOriginAction->setOn(false);
		ActionViewOrigin();
	}

	if (MsgOperation != MSG_OPERATION_UNSUBSCRIBE)
	{
		QStringList UserLst;
		Config->AddressBook->MakeUserLst(&UserLst);
		if (UserLst.count() > 0)
			ToNameCombo->insertStringList(UserLst);
	}
	ToNameCombo->setCurrentItem(0);

	SetupDone = true;
	return true;
}

void TEditWindow::LoadTemplate(QString Name, QPoint *pos)
{
	int p;
	QPoint ret(0, 0);
	QString tmp, Line;

	tmp = Config->ConfigPath + Name + "." + DestArea;
	if (!testexists(tmp))
	{
		tmp = Config->ConfigPath + Name;
		if (!testexists(tmp))
			return;
	}

	QFile tpl(tmp);
	if (FileOpenFunc(&tpl, IO_ReadOnly))
	{
		QTextStream stream(&tpl);
		while (!stream.atEnd())
		{
			Line = stream.readLine().append("\n");
			p = Line.find("@cursor@");
			if (p > -1)
			{
				pos->setX(p);
				pos->setY(MessageEdit->paragraphs() - 1);
				Line.replace("@cursor@", QString::null);
			}
			MessageEdit->append(Line);
		}
		FileCloseFunc(&tpl);
	}
}

void TEditWindow::LoadKludges(QStringList *Kludges)
{
	QString tmp, Line;
	QStringList ret;

	tmp = Config->ConfigPath + TPL_KLUDGES_LST + "." + DestArea;
	if (!testexists(tmp))
	{
		tmp = Config->ConfigPath + TPL_KLUDGES_LST;
		if (!testexists(tmp))
			return;
	}

	QFile tpl(tmp);
	if (FileOpenFunc(&tpl, IO_ReadOnly))
	{
		QTextStream stream(&tpl);
		while (!stream.atEnd())
		{
			Line = stream.readLine().stripWhiteSpace();
			if (tokencount(Line, ": ") > 1)
			{
				ret = Kludges->grep(QRegExp("^" + Line, false));
				if (ret.count() == 0)
					Kludges->append(Line);
			}
		}
		FileCloseFunc(&tpl);
	}
}

QString TEditWindow::ComposeOrigin(void *_addr)
{
	ps_addr addr = (ps_addr)_addr;
	QString origline, addrstr = QString("%1:%2/%3").arg(addr->zone).arg(addr->net).arg(addr->node);

	if (addr->point > 0)
		addrstr.append(QString(".%1").arg(addr->point));
	addrstr.prepend(" (").append(")");

	if (OriginCombo->isShown() && OriginCombo->listBox()->count() > 2)
		origline = OriginCombo->text(RandomNumber(1, OriginCombo->listBox()->count() - 1)).stripWhiteSpace();
	else
		origline = OriginCombo->currentText().stripWhiteSpace();

	if (origline.isEmpty())
		return QString::null;

	while (ParseCommand(this, &origline)) {};
	if (origline.length() > (68 - addrstr.length()))
		origline.setLength(68 - addrstr.length());

	return QString(" * Origin: %1%2\r").arg(origline).arg(addrstr);
}

void TEditWindow::ParseTemplate(TArea *Area, TMessage *Message, bool isFinal)
{
	QString Line = MessageEdit->text();

	Line.replace("@areadesc@", Area->Desc);
	Line.replace("@area@", DestArea);
	if (!isFinal)
		ExpandMacros(Area, Message, &Line, OrigArea);
	else {
		Line.replace("@to@", ToNameCombo->currentText());
		Line.replace("@toname@", gettoken(ToNameCombo->currentText(), 1));
		if (Area->Type == AREA_TYPE_NETMAIL)
			Line.replace("@toaddr@", ToAddrEdit->text());
		Line.replace("@from@", FromNameCombo->currentText());
		Line.replace("@fromname@", gettoken(FromNameCombo->currentText(), 1));
		Line.replace("@fromaddr@", FromAddrCombo->currentText());
		Line.replace("@subject@", SubjEdit->text());
		QDateTime dt = QDateTime::currentDateTime();
		Line.replace("@date@", dt.toString("dd.MM.yyyy"));
		Line.replace("@time@", dt.toString("hh:mm:ss"));
		Line.replace("@attr@", flags2str(MessFlags, NULL));
		ExpandMacros2(this, &Line);
	}
	MessageEdit->setText(Line);
}

int TEditWindow::LoadText(TMessage *Message)
{
	int q;
	uint i, j;

	char *TextBuffer = new char[Message->TextLen + 1];
	qstrncpy(TextBuffer, (char*)Message->TextBuffer, Message->TextLen);
	TextBuffer[Message->TextLen] = '\0';
	fix_crlf(TextBuffer);
	QStringList NameList, InputList = QStringList::split("\n", Config->toUTF(TextBuffer), true);
	QString Initials, Line, Text = MessageEdit->text();

	if (InputList.count())
		while (InputList[InputList.count() - 1].startsWith("SEEN-BY: ") || InputList[InputList.count() - 1].startsWith("\001PATH: ") || InputList[InputList.count() - 1].startsWith("\001Via "))
			InputList.remove(InputList[InputList.count() - 1]);

	if ((MsgOperation == MSG_OPERATION_REPLY) || (MsgOperation == MSG_OPERATION_REPLYNET))
	{
		NameList = QStringList::split(' ', (char*)Message->MsgInfo.from);
		Initials = QString::null;
		for (i = 0; i < QMIN(NameList.count(), 3); i++)
			if (!NameList[i].isEmpty())
				if (NameList[i].at(0).isLetter())
					Initials += NameList[i].at(0);
		for (i = 0; i < InputList.count(); i++)
		{
			Line = InputList[i];
			q = Line.find('>');
			if (q != -1)
			{
				if (q < 5)
				{
					for (j = q + 1; j < Line.length(); j++)
						if (Line.at(j) == '>')
							q++;
						else
							break;
					Line.insert(q, ">");
				} else
					Line.prepend(Initials + "> ");
			} else
				Line.prepend(" " + Initials + "> ");
			InputList[i] = Line;
		}
		Text.replace("@quotetext@", InputList.join("\n"));
	} else
		Text.replace("@text@", InputList.join("\n"));

	MessageEdit->setText(Text);
	return ((InputList.count() > 1) ? (InputList.count() - 1) : 0);
}

void TEditWindow::destroy()
{
	delete EditActions;
}

void TEditWindow::showEvent(QShowEvent *)
{
	if (FirstTimeInit)
		return;

	switch (MsgOperation)
	{
		case MSG_OPERATION_EDIT:
		case MSG_OPERATION_REPLY:
		case MSG_OPERATION_REPLYNET:
		case MSG_OPERATION_UNSUBSCRIBE:
			MessageEdit->setFocus();
			break;
		default:
			break;
	}

	PasteTest();
	connect(qApp->clipboard(), SIGNAL(dataChanged()), this, SLOT(PasteTest()));
}

void TEditWindow::closeEvent(QCloseEvent *e)
{
	if (MessageEdit->isModified())
		if (!questionmessage(this, QObject::tr("Do You really want Quit without Save Message?")))
			return;

	disconnect(qApp->clipboard(), SIGNAL(dataChanged()), this, SLOT(PasteTest()));

	Config->SetToolBar(this, EditToolBar);
	Config->SetBool(CONFIG_SECTION_EDIT, CONFIG_VIEWSTATUSBAR, statusBar()->isShown());
	Config->SetBool(CONFIG_SECTION_EDIT, CONFIG_VIEWORIGIN, OriginCombo->isShown());

	Config->SetPos(this);
	Config->SetSize(this);
	if (SetupDone)
		qApp->postEvent(parent(), new TCloseEvent(this, Result));
	e->accept();
}

void TEditWindow::ClearModified()
{
	MessageEdit->setModified(false);

	MessageEdit->setUndoRedoEnabled(false);
	MessageEdit->setUndoRedoEnabled(true);

	SaveMessageAction->setEnabled((MsgOperation == MSG_OPERATION_FORWARD) || (MsgOperation == MSG_OPERATION_UNSUBSCRIBE));
	SaveDraftAction->setEnabled(MsgOperation == MSG_OPERATION_FORWARD);

	RedoAction->setEnabled(false);
	UndoAction->setEnabled(false);
	CopyAction->setEnabled(false);
	CutAction->setEnabled(false);
	DelAction->setEnabled(false);
}

void TEditWindow::ToAddrChanged(const QString &s)
{
	int pos = 0;
	ToAddrEdit->setPaletteForegroundColor(ToAddrEdit->validator()->validate((QString&)s, pos) == QValidator::Acceptable ? Qt::black : Qt::red);
}

QString TEditWindow::TextReplace(QString Text)
{
	QStringList Replace;

	Config->GetStrList(CONFIG_SECTION_EDIT, CONFIG_REPLACELIST, &Replace);
	for (uint i = 0; i < Replace.count(); i++)
		if (tokencount(Replace[i], "\t") == 2)
			Text.replace(gettoken(Replace[i], 1, "\t"), gettoken(Replace[i], 2, "\t"));

	return Text;
}

void TEditWindow::Save(bool isDraft)
{
	uint tmpIdx;
	TArea *Area, *Draft = NULL;
	TMessage *Message = NULL;
	time_t curtime = time(NULL);
	struct tm *tm;
	QString tmp;
	QCString _tmp;

	if (!Config->GetDraftArea().isEmpty())
	{
		tmpIdx = Areas->Find(Config->GetDraftArea());
		if (tmpIdx == 0)
		{
			errormessage(this, MESS_NOTAREA);
			return;
		}
		Draft = Areas->at(tmpIdx - 1);
	}

	if (!isDraft)
	{
		tmpIdx = Areas->Find(DestArea);
		if (tmpIdx == 0)
		{
			errormessage(this, MESS_NOTAREA);
			return;
		}
		Area = Areas->at(tmpIdx - 1);
	} else
		Area = Draft;

	if ((MsgOperation == MSG_OPERATION_EDIT) && !strcompare(OrigArea, Config->GetDraftArea()))
	{
		tmpIdx = Area->UID2Index(UniqueID);
		if (tmpIdx == 0)
		{
			errormessage(this, MESS_NOTMESSAGE);
			return;
		}
		Message = Area->at(tmpIdx - 1);
	} else
		Message = Area->New();

	ps_addr faddr = getAddr(Config->FidoConfig, (char*)FromAddrCombo->currentText().ascii());
	hs_addr taddr;

	if (faddr == NULL)
	{
		errormessage(this, QObject::tr("Sender address is invalid. Please, select valid address."));
		return;
	}

	if (Area->Type == AREA_TYPE_ECHOMAIL)
		memcpy(&taddr, faddr, sizeof(hs_addr));
	else
	{
		string2addr((char*)ToAddrEdit->text().ascii(), &taddr);
		if (taddr.zone == 0)
		{
			errormessage(this, QObject::tr("Receiver address is invalid. Please, select valid address."));
			return;
		}
	}

	tmp = SubjEdit->text();
	if (tmp.stripWhiteSpace().isEmpty())
	{
		errormessage(this, QObject::tr("Message Subject is Empty."));
		return;
	}

	if (!isDraft)
		ParseTemplate(Area, Message, true);
	tmp = QString("%1:%2/%3").arg(faddr->zone).arg(faddr->net).arg(faddr->node);
	if (faddr->point > 0)
		tmp.append(QString(".%1").arg(faddr->point));
	if (faddr->domain != NULL)
		tmp.append(QString("@%1").arg(faddr->domain));
	tmp.prepend("MSGID: ").append(" " + GetMessageMsgId());
	if (((MsgOperation == MSG_OPERATION_NEW) || (MsgOperation == MSG_OPERATION_UNSUBSCRIBE)) && (Area->Type == AREA_TYPE_NETMAIL))
		Kludges.prepend(QString("INTL %1:%2/%3 %4:%5/%6").arg(taddr.zone).arg(taddr.net).arg(taddr.node).arg(faddr->zone).arg(faddr->net).arg(faddr->node));
	Kludges.prepend(tmp);

	if (!isDraft)
		Kludges.prepend(QString("PID: QFE/%1 %2 (http://qfe.sourceforge.net/)").arg(PLATFORM_NAME).arg(QFE_VERSION));

	if (MessFlags & MSGXX2)
		tmp = "DIR";
	else
		tmp = QString::null;
	if (MessFlags & MSGIMM)
		tmp += (tmp.isEmpty() ? QString::null : " ") + "IMM";
	if (!tmp.isEmpty())
		Kludges.append(QString("FLAGS %1").arg(tmp));
	tmp = Config->CharsetForMessages();
	if (!tmp.isEmpty())
		Kludges.append(QString("CHRS: %1").arg(tmp));

	addr2addr(&Message->MsgInfo.orig, faddr);
	addr2addr(&Message->MsgInfo.dest, &taddr);
	qstrcpy((char*)Message->MsgInfo.from, FromNameCombo->currentText().ascii());
	qstrcpy((char*)Message->MsgInfo.to, ToNameCombo->currentText().ascii());
	tmp = isDraft ? SubjEdit->text().stripWhiteSpace() : TextReplace(SubjEdit->text().stripWhiteSpace());
	_tmp = Config->fromUTF(tmp);
	qstrcpy((char*)Message->MsgInfo.subj, _tmp);

	Message->MsgInfo.attr = MessFlags | MSGREAD;
	tm = localtime(&curtime);
	fts_time((char*)Message->MsgInfo.__ftsc_date, tm);

	Message->MsgInfo.date_written.date.da = tm->tm_mday;
	Message->MsgInfo.date_written.date.mo = tm->tm_mon + 1;
	Message->MsgInfo.date_written.date.yr = tm->tm_year - 80;
	Message->MsgInfo.date_written.time.hh = tm->tm_hour;
	Message->MsgInfo.date_written.time.mm = tm->tm_min;
	Message->MsgInfo.date_written.time.ss = tm->tm_sec >> 1;

	Message->MsgInfo.date_arrived.date.da = tm->tm_mday;
	Message->MsgInfo.date_arrived.date.mo = tm->tm_mon + 1;
	Message->MsgInfo.date_arrived.date.yr = tm->tm_year - 80;
	Message->MsgInfo.date_arrived.time.hh = tm->tm_hour;
	Message->MsgInfo.date_arrived.time.mm = tm->tm_min;
	Message->MsgInfo.date_arrived.time.ss = tm->tm_sec >> 1;

	LoadKludges(&Kludges);
	tmp = Kludges.join("\x1").prepend("\x1");
	_tmp = Config->fromUTF(tmp.replace('\r', QString::null));

	Message->CtrlLen = qstrlen(_tmp) + 1;
	Message->CtrlBuffer = (byte*)malloc(Message->CtrlLen + 1);
	*(Message->CtrlBuffer + Message->CtrlLen) = '\0';
	qstrncpy((char*)Message->CtrlBuffer, _tmp, Message->CtrlLen);

	tmp = isDraft ? MessageEdit->text() : TextReplace(MessageEdit->text());
	if (isDraft)
		tmp.prepend(QString("\001AREA: %1\r").arg(DestArea));
	else if ((MsgOperation != MSG_OPERATION_EDIT) && (MsgOperation != MSG_OPERATION_UNSUBSCRIBE))
	{
		if (!tmp.endsWith("\r") && !tmp.endsWith("\n"))
			tmp.append("\r");
		if ((Area->Type == AREA_TYPE_NETMAIL) && (tmp.find("---") < 0))
			tmp.append("---\r");
		tmp.append(isDraft ? ComposeOrigin(faddr) : TextReplace(ComposeOrigin(faddr)));
	}

	_tmp = Config->fromUTF(tmp);

	Message->TextLen = qstrlen(_tmp) + 1;
	Message->TextBuffer = (byte*)malloc(Message->TextLen + 1);
	*(Message->TextBuffer + Message->TextLen) = '\0';
	qstrncpy((char*)Message->TextBuffer, _tmp, Message->TextLen);

	unfix_crlf((char*)Message->TextBuffer);

	if (!Message->WriteMessage(true))
	{
		errormessage(this, QObject::tr("Can't Write Message!"));
		return;
	} else if ((MsgOperation == MSG_OPERATION_EDIT) && !isDraft && strcompare(OrigArea, Config->GetDraftArea()))
	{
		tmpIdx = Draft->UID2Index(UniqueID);
		if (tmpIdx == 0)
		{
			errormessage(this, QObject::tr("Can't Delete Draft Message!"));
			return;
		} else
			Draft->KillMessage(tmpIdx);
	}

	ClearModified();
	if ((Config->FidoConfig->echotosslog != NULL) && !isDraft)
	{
		bool found = false;
		QFile echotoss(Config->FidoConfig->echotosslog);
		if (FileOpenFunc(&echotoss, IO_ReadWrite | IO_Append))
		{
			echotoss.reset();
			QTextStream stream(&echotoss);
			while (!stream.atEnd())
			{
				tmp = stream.readLine();
				if (strcompare(tmp, Message->Area->Name) && !found)
					found = true;
			}
			if (!found)
				stream << Area->Name << "\n";
			FileCloseFunc(&echotoss);
		}
	}
	Result = true;
	WidgetCloseFunc(this);
}

void TEditWindow::ActionClose()
{
	WidgetCloseFunc(this);
}

void TEditWindow::ActionSaveMessage()
{
	Save(false);
}

void TEditWindow::ActionSaveDraft()
{
	Save(true);
}

void TEditWindow::ActionViewToolBar()
{
	EditToolBar->setShown(ViewToolBarAction->isOn());
}

void TEditWindow::ActionViewStatusBar()
{
	statusBar()->setShown(ViewStatusBarAction->isOn());
}

void TEditWindow::ToolBarVisibilityChanged(bool)
{
	if (isShown())
		ViewToolBarAction->setOn(EditToolBar->isShown());
}

void TEditWindow::ActionViewOrigin()
{
	OriginCombo->setShown(ViewOriginAction->isOn());
	OriginLabel->setShown(ViewOriginAction->isOn());
}

void TEditWindow::ActionHelp()
{
	((TMainWindow*)qApp->mainWidget())->HelpIndex();
}

void TEditWindow::ActionWhatsThis()
{
	QWhatsThis::enterWhatsThisMode();
}

void TEditWindow::MessageTextChanged()
{
	if (isShown())
	{
		SaveMessageAction->setEnabled(true);
		SaveDraftAction->setEnabled((MsgOperation != MSG_OPERATION_UNSUBSCRIBE) && !Config->GetDraftArea().isEmpty());
	}
}

void TEditWindow::MessageEditRedoAvailable(bool Val)
{
	RedoAction->setEnabled(Val);
}

void TEditWindow::MessageEditUndoAvailable(bool Val)
{
	UndoAction->setEnabled(Val);
}

void TEditWindow::MessageEditCopyAvailable(bool Val)
{
	CopyAction->setEnabled(Val);
	CutAction->setEnabled(Val);
	DelAction->setEnabled(Val);
}

void TEditWindow::PasteTest()
{
	PasteAction->setEnabled(!qApp->clipboard()->text().isEmpty());
}

void TEditWindow::ActionSpellCheck()
{
	qDebug("TEditWindow::ActionSpellCheck() Not implemented yet...");
}

void TEditWindow::ActionAttributes()
{
	TMessAttr *AttrWnd = new TMessAttr(this);
	AttrWnd->SetUp(AllowFlags, MessFlags);
	if (AttrWnd->exec() == QDialog::Accepted)
		MessFlags = AttrWnd->GetState();
	delete AttrWnd;
}

void TEditWindow::ActionInsertFile()
{
	QFileDialog* fd = new QFileDialog(this);
	fd->setDir(Config->GetStr(QString::null, CONFIG_LASTOPENSAVE, QString::null));
	fd->setFilter(QObject::tr("Text Files (*.txt)"));
	fd->addFilter(QObject::tr("All Files (*)"));
	fd->setSelectedFilter(0);
	fd->setCaption(QObject::tr("Select file..."));
	fd->setIcon(*Config->FindPixmap("icon_load.xpm"));
	fd->setMode(QFileDialog::ExistingFile);
	fd->setInfoPreviewEnabled(true);
	if (fd->exec() == QDialog::Accepted)
	{
		Config->SetStr(QString::null, CONFIG_LASTOPENSAVE, fd->dir()->absPath());

		int mpara, mindex;
		QFile txtfile(fd->selectedFile());
		if (FileOpenFunc(&txtfile, IO_ReadOnly))
		{
			if (txtfile.size() > 0)
			{
				QTextStream stream(&txtfile);
				MessageEdit->getCursorPosition(&mpara, &mindex);
				MessageEdit->insertParagraph(TextStreamReadFunc(&stream), mpara + ((mindex > 0) ? 1 : 0));
			}
			FileCloseFunc(&txtfile);
		}
	}
	delete fd;
}

void TEditWindow::ActionAttachFile()
{
	qDebug("TEditWindow::ActionAttachFile() Not implemented yet...");
}

void TEditWindow::ActionAddrBook()
{
        TAddressBook *AddressBook = new TAddressBook(this);
        AddressBook->SetUp(true);
        if (AddressBook->exec() == QDialog::Accepted)
        {
		ToNameCombo->insertItem(AddressBook->Selected->Name, 0);
		ToNameCombo->setCurrentItem(0);
		if (ToAddrEdit->isEnabled())
			ToAddrEdit->setText(AddressBook->Selected->Address);
	}
        delete AddressBook;
}

void TEditWindow::ActionNodeListBrowser()
{
/*
	TNodelistBrowser *NodelistBrowser = new TNodelistBrowser(this);
	NodelistBrowser->SetSelectMode(true);
	if (NodelistBrowser->exec() == QDialog::Accepted)
	{
		//
	}
	delete NodelistBrowser;
*/
	qDebug("TeDITWindow::ActionNodelistBrowser() Not implemented yet...");
}

void TEditWindow::ActionUndo()
{
	MessageEdit->undo();
}

void TEditWindow::ActionRedo()
{
	MessageEdit->redo();
}

void TEditWindow::ActionCopy()
{
	MessageEdit->copy();
}

void TEditWindow::ActionCut()
{
	MessageEdit->cut();
}

void TEditWindow::ActionDel()
{
	MessageEdit->del();
}

void TEditWindow::ActionPaste()
{
	MessageEdit->paste();
}

bool TEditWindow::eventFilter(QObject *o, QEvent *e)
{
        if (o == MessageEdit)
        {
                if (e->type() == QEvent::Paint)
		{
			//TODO: Draw line at column 80
			//QPaintEvent *event = (QPaintEvent*)e;

			//QWidget::paintEvent(event);
			//QPainter paint(MessageEdit);
			//paint.setPen(Qt::red);
			//paint.drawLine(0,0,20,20);
			//return true;
		}
	}
	return false;
}
