/***************************************************************************
 *   Copyright (C) 2005 by Alexander Shiyan                                *
 *   shc@users.sourceforge.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "utils.h"

extern TConfig *Config;

// Viewed Value, QTextCodec Value, CHRS Value
QString QFECodePages[CODEPAGES_COUNT][3] = {
	{"ISO8859-1", "ISO8859-1", "LATIN-1"},
	{"IBM 850", "IBM 850", "CP850 2"},
	{"IBM 866", "IBM 866", "CP866 2"},
	{"UTF-8", "utf8", "UTF-8 4"}
};

void errormessage(QWidget *parent, QString Message)
{
	QMessageBox::critical(parent, QObject::tr("Error"), Message, QObject::tr("OK"));
}

void informationmessage(QWidget *parent, QString Message)
{
	QMessageBox::information(parent, QObject::tr("Information"), Message, QObject::tr("OK"));
}

bool questionmessage(QWidget *parent, QString Message)
{
	return (QMessageBox::question(parent, QObject::tr("Question"), Message, QObject::tr("Yes"), QObject::tr("No"), QString::null, 1, 1) == 0);
}

QString addr2str1(NETADDR _addr, byte *CtrlBuff)
{
	QString ret = QString("%1:%2/%3").arg(_addr.zone).arg(_addr.net).arg(_addr.node);
	ret.append(_addr.point ? QString(".%1").arg(_addr.point) : QString::null);

	if (CtrlBuff && (_addr.zone == 0) && (_addr.net == 0) && (_addr.node == 0))
	{
		char *msgi = (char*)GetCtrlToken(CtrlBuff, (byte *)"MSGID: ");
		if (msgi)
		{
			ret = gettoken(msgi + 7, 1);
			nfree(msgi);
		}
	}

	return ret;
}

QString addr2str2(hs_addr _addr)
{
	return QString(aka2str(_addr));
}

void addr2addr(NETADDR *_addr1, hs_addr *_addr2)
{
	_addr1->zone = _addr2->zone;
	_addr1->net = _addr2->net;
	_addr1->node = _addr2->node;
	_addr1->point = _addr2->point;
}

QString flags2str(dword flags, char *line)
{
	QString ext, ret = QString::null;

	if (flags & MSGPRIVATE) ret.append("Pvt ");
	if (flags & MSGCRASH) ret.append("Cra ");
	if (flags & MSGSENT) ret.append("Snt ");
	if (flags & MSGFILE) ret.append("Att ");
	if (flags & MSGFWD) ret.append("Fwd ");
	if (flags & MSGORPHAN) ret.append("Orp ");
	if (flags & MSGKILL) ret.append("K/S ");
	if (flags & MSGLOCAL) ret.append("Loc ");
	if (flags & MSGHOLD) ret.append("Hld ");
	if (flags & MSGXX2) ret.append("Dir ");
	if (flags & MSGFRQ) ret.append("Frq ");
	if (flags & MSGRRQ) ret.append("Rrq ");
	if (flags & MSGCPT) ret.append("Irq ");
	if (flags & MSGARQ) ret.append("Arq ");
	if (flags & MSGURQ) ret.append("Urq ");
	if (flags & MSGIMM) ret.append("Imm ");
	if (line != NULL)
	{
		ext = line;
		if ((ext.find("PVT") > -1) && ((flags & MSGPRIVATE) == 0))
			ret.append("Pvt ");
		if ((ext.find("HLD") > -1) && ((flags & MSGHOLD) == 0))
			ret.append("Hld ");
		if ((ext.find("CRA") > -1) && ((flags & MSGCRASH) == 0))
			ret.append("Cra ");
		if ((ext.find("K/S") > -1) && ((flags & MSGKILL) == 0))
			ret.append("K/S ");
		if ((ext.find("SNT") > -1) && ((flags & MSGSENT) == 0))
			ret.append("Snt ");
		if (ext.find("RCV") > -1)
			ret.append("Rcv ");
		if (ext.find("A/S") > -1)
			ret.append("A/S ");
		if ((ext.find("DIR") > -1) && ((flags & MSGXX2) == 0))
			ret.append("Dir ");
		if (ext.find("ZON") > -1)
			ret.append("Zon ");
		if (ext.find("HUB") > -1)
			ret.append("Hub ");
		if ((ext.find("FIL") > -1) && ((flags & MSGFILE) == 0))
			ret.append("Att ");
		if ((ext.find("FRQ") > -1) && ((flags & MSGFRQ) == 0))
			ret.append("Frq ");
		if ((ext.find("IMM") > -1) || (ext.find("FPU") > -1))
			ret.append("Imm ");
		if ((ext.find("RRQ") > -1) && ((flags & MSGRRQ) == 0))
			ret.append("Rrq ");
		if ((ext.find("CFM") > -1) && ((flags & MSGCPT) == 0))
			ret.append("Irq ");
		if (ext.find("FAX") > -1)
			ret.append("Fax ");
		if (ext.find("NPD") > -1)
			ret.append("Npd ");
	}
	return ret.stripWhiteSpace();
}

QString flag2desc(dword flag)
{
	if (flag == MSGPRIVATE)
		return QObject::tr("Indicates that the message may only be read by its addressee and author.");
	if (flag == MSGCRASH)
		return QObject::tr("High-priority mail.");
	if (flag == MSGSENT)
		return QObject::tr("Message has been successfully sent.");
	if (flag == MSGFILE)
		return QObject::tr("Message has one or more files attached to it.");
	if (flag == MSGFWD)
		return QObject::tr("Message is not destined for this system.");
	if (flag == MSGORPHAN)
		return  QObject::tr("Unknown destination.");
	if (flag == MSGKILL)
		return QObject::tr("Remove message after it has been sent.");
	if (flag == MSGLOCAL)
		return QObject::tr("Message is created locally.");
	if (flag == MSGHOLD)
		return QObject::tr("Message should be held for pickup by its destination system.");
	if (flag == MSGXX2)
		return QObject::tr("Message must be sent directly to its destination and may not be routed.");
	if (flag == MSGFRQ)
		return QObject::tr("Message has one or more file requests in subject field.");
	if (flag == MSGRRQ)
		return QObject::tr("Return receipt request.");
	if (flag == MSGCPT)
		return QObject::tr("Is Return Receipt.");
	if (flag == MSGARQ)
		return QObject::tr("Audit Request.");
	if (flag == MSGURQ)
		return QObject::tr("Update Request.");
	if (flag == MSGIMM)
		return QObject::tr("Immediate mail (highest priority).");
	return QString::null;
}

void fix_crlf(char *buffer)
{
	char *tmp = buffer;

	while (*buffer)
	{
		if (*buffer == 0x0d)
			*tmp++ = '\n';
		else if (*buffer != 0x0a)
			*tmp++ = *buffer;
		buffer++;
	}
	*tmp = '\0';
}

void unfix_crlf(char *buffer)
{
	char *tmp = buffer;

	while (*buffer)
	{
		if (*buffer == 0x0a)
			*tmp++ = 0x0d;
		else
			*tmp++ = *buffer;
		buffer++;
	}
	*tmp = '\0';
}

QString convert2qhtm(QString buffer)
{
	return buffer.replace('\n', "<br>").replace(' ', "&nbsp;").replace("<a&nbsp;href=", "<a href=");
}

QString convert2qhtm_ext(QString buffer)
{
	return buffer.replace('<', "&lt;").replace('>', "&gt;");
}

bool testexists(QString Name)
{
	bool ret = false;
	QFile File(Name);
	if (File.exists())
		if (FileOpenFunc(&File, IO_ReadOnly))
		{
			ret = true;
			FileCloseFunc(&File);
		}
	return ret;
}

bool strcompare(QString str1, QString str2)
{
	return (qstricmp(str1.ascii(), str2.ascii()) == 0);
}

int tokencount(QString str, QString sep)
{
	QString tmp = str.stripWhiteSpace();
	uint j;
	int i, k, ret;

	ret = tmp.isEmpty() ? 0 : 1;

	while (true)
	{
		i = -1;
		for (j = 0; j < sep.length(); j++)
		{
			k = tmp.find(sep.at(j));
			if (k > -1)
				i = (i > -1) ? QMIN(i, k) : k;
		}

		if (i > -1)
		{
			tmp = tmp.mid(i + 1).stripWhiteSpace();
			if (!tmp.isEmpty())
				ret++;
			else
				break;
		} else
			break;
	}

	return ret;
}

QString gettoken(QString str, int num, QString sep)
{
	QString ret = QString::null, tmp = str.stripWhiteSpace();
	uint j;
	int i, k, cnt = 0;

	while (true)
	{
		i = -1;
		for (j = 0; j < sep.length(); j++)
		{
			k = tmp.find(sep.at(j));
			if (k > -1)
				i = (i > -1) ? QMIN(i, k) : k;
		}

		if (i > -1)
		{
			if (cnt == (num - 1))
			{
				if (i > 0)
					tmp.truncate(i);
				ret = tmp;
				break;
			}
			tmp = tmp.mid(i + 1).stripWhiteSpace();
			if (!tmp.isEmpty())
				cnt++;
			else
				break;
		} else {
			if (cnt == (num - 1))
				ret = tmp;
			break;
		}
	}

	return ret;
}

void AppendArray(QByteArray *target, QByteArray *source)
{
	uint offset = target->size();
	target->resize(offset + source->size());
	for (uint i = 0; i < source->size(); i++)
		target->at(offset + i) = source->at(i);
}

bool ParseCommand(QObject *parent, QString *val)
{
	QString cmd;
	int i, tc, end, start = val->find("@command ", 0, false);

	if (start > -1)
	{
		end = val->find("@", start + 9, false);
		if (end > -1)
		{
			cmd = val->mid(start + 9, end - (start + 9));
			val->remove(start, (end + 1) - start);
			QProcess proc(parent);
			tc = tokencount(cmd, " \t");
			if (tc > 0)
			{				
				for (i = 0; i < tc; i++)
					proc.addArgument(gettoken(cmd, i + 1, " \t"));

				proc.setWorkingDirectory(QDir());
				if (proc.start())
				{
					QTime tim;
					tim.start();
					while (proc.isRunning() && (tim.elapsed() < 10000))
						qApp->processEvents();
					if (proc.isRunning())
					{
						proc.tryTerminate();
						QTimer::singleShot(1000, &proc, SLOT(kill()));
						fprintf(stdout, "Command %s terminated.\n", cmd.ascii());
					} else if (proc.normalExit()) {
						cmd = QString::fromLocal8Bit((const char*)proc.readStdout().data());
						if (!cmd.isEmpty())
							val->insert(start, cmd);
					} else
						fprintf(stdout, "Command %s. Error returned.\n", cmd.ascii());
				} else
					fprintf(stdout, "Can`t start command: %s.\n", cmd.ascii());
			}
			return true;
		}
	}
	return false;
}

bool ValidateFileName(QString fn)
{
	if (Config->FidoConfig->notValidFNChars)
		for (uint i = 0; i < strlen(Config->FidoConfig->notValidFNChars); i++)
			if (fn.find(QChar(*(Config->FidoConfig->notValidFNChars + i))) > -1)
				return false;

	return true;
}

int CompareVersions(QString ver1, QString ver2)
{
	int ret, val1, val2, cnt = 2;
	bool ok1, ok2;
	QString tok1, tok2;

	tok1 = gettoken(ver1, 1, "._- ");
	tok2 = gettoken(ver2, 1, "._- ");

	while (!tok1.isEmpty() && !tok2.isEmpty())
	{
		val1 = tok1.toInt(&ok1);
		val2 = tok2.toInt(&ok2);
		if (ok1 && ok2)
		{
			if (val1 > val2)
				return 1;
			if (val1 < val2)
				return -1;
		} else {
			ret = qstricmp(tok1.ascii(), tok2.ascii());
			if (ret != 0)
				return ret;
		}

		tok1 = gettoken(ver1, cnt, "._- ");
		tok2 = gettoken(ver2, cnt, "._- ");
		cnt++;
	}

	return (tok2.isEmpty() ? (tok1.isEmpty() ? 0 : 1) : -1);
}

QString GetMessageMsgId()
{
	if (Config->FidoConfig->seqDir && (Config->FidoConfig->seqOutrun > 0))
	{
		dword id = GenMsgId(Config->FidoConfig->seqDir, Config->FidoConfig->seqOutrun);
		if (id != 0)
			return QString("%1").arg(id).rightJustify(8, '0', true);
	}

	time_t curtime = time(NULL);
	return QString("%1").arg(curtime, 8, 16).rightJustify(8, '0', true);
}

uint RandomNumber(uint minval, uint maxval)
{
	uint rnd;

	for (;;) {
		rnd = (uint)random() % (maxval + 1);
		if ((rnd >= minval) && (rnd <= maxval))
			break;
	}

	return rnd;
}

void ExpandMacros(TArea *Area, TMessage *Message, QString *Line, QString _Area)
{
	if (Message != NULL)
	{
		Line->replace("@origarea@", _Area);
		Line->replace("@origareadesc@", Message->Area->Desc);
		Line->replace("@origto@", (char*)Message->MsgInfo.to);
		Line->replace("@origtoname@", gettoken((char*)Message->MsgInfo.to, 1));
		if (Area->Type == AREA_TYPE_NETMAIL)
			Line->replace("@origtoaddr@", addr2str1(Message->MsgInfo.dest));
		Line->replace("@origfrom@", (char*)Message->MsgInfo.from);
		Line->replace("@origfromname@", gettoken((char*)Message->MsgInfo.from, 1));
		Line->replace("@origfromaddr@", addr2str1(Message->MsgInfo.orig, Message->CtrlBuffer));
		Line->replace("@origsubj@", Config->toUTF((char*)Message->MsgInfo.subj));
		QDateTime dt(QDate(Message->MsgInfo.date_written.date.yr + 1980, Message->MsgInfo.date_written.date.mo, Message->MsgInfo.date_written.date.da), QTime(Message->MsgInfo.date_written.time.hh, Message->MsgInfo.date_written.time.mm, Message->MsgInfo.date_written.time.ss));
		Line->replace("@origdate@", dt.toString("dd.MM.yyyy"));
		Line->replace("@origtime@", dt.toString("hh:mm:ss"));
		Line->replace("@origattr@", flags2str(Message->MsgInfo.attr, (char*)GetCtrlToken(Message->CtrlBuffer, (byte *)"FLAGS")));
	}
}

void ExpandMacros2(QWidget *Widget, QString *Line)
{
	Line->replace("@product@", "QFE");
	Line->replace("@platform@", PLATFORM_NAME);
	Line->replace("@version@", QFE_VERSION);

	while (ParseCommand(Widget, Line))
		qApp->processEvents();
}
